#include "batchform.h"
#include "presetthumbnail.h"
#include "batchsettingswindow.h"
#include "openimageform.h"
#include <fastformat/fastformat.hpp>
#include <iostream>
#include <iomanip>
#include <gtkmm-3.0/gtkmm/filechooserdialog.h>

#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <unistd.h>

#include "../PhotomatixImaging/photoimage.h"
#include "../PhotomatixImaging/photoprocessor.h"
#include "../Photomatix32/Photomatix32.h"
#include "../Photomatix32/PhotomatixMetaLib.h"
#include "../Photomatix32/MD5.h"
#include "../PhotomatixMisc/appproperties.h"
#include "../PhotomatixMisc/stringutils.h"
#include "../PhotomatixMisc/localtext.h"

using namespace std;
using namespace PhotomatixImaging;
using namespace PhotomatixMisc;


namespace Photomatix
{
	BatchProcessor::BatchProcessor(int setsNum, int setSize, std::vector <std::string>& cmd, std::vector <std::string>& files,
								   Glib::RefPtr<Gtk::TextBuffer> textBuf, BatchForm* form, int steps):
		command(cmd),
		fileNames(files)
	{
		this->bracketSize = setSize;
		this->totalNumberOfSets = setsNum;
		this->textBuffer = textBuf;
		this->batchForm = form;
		this->totalSteps = steps * setsNum;
		this->doneSteps = 0;
	}
	
	bool BatchProcessor::TreatBatchOutput(Glib::IOCondition condition)
	{
		if ((condition & Glib::IO_IN) == 0)
		{
			return true;
		}
		
		Glib::ustring buf;
		
		//if(this->ch_stdout->read_line(buf) != Glib::IO_STATUS_NORMAL)
//			return false;
		if(!this->ch_stdout) return true;
		
		this->ch_stdout->read_line(buf);
		
		std::vector<std::string> strs = PhotomatixUtils::split(buf, ' ');
		int code = -1;
		int subcode = -1;
		string msg = "";
		if(strspn(strs[0].c_str(), "-.0123456789") > 0)
		{
			code = atoi(strs[0].c_str());
			if(strs.size() > 1) subcode = atoi(strs[1].c_str());
			
		
			if ((code>=1500)&&(code<2000)) {
				//this.stripNumber=code-1500;
				code=1500; //CodeStripProcess
			}
			
			const int CodeProcessRaw=1001;
			const int CodeNoiseReduction = 1005;
			const int CodeAlign=1010;
			const int CodeGenerateHdr=1103;
			const int CodeStripGenerateHdr=1110;
			const int CodeStripProcess=1500;
			const int CodeToneMapDE=1201;
			const int CodeToneMapTC=1202;
			const int CodeToneMapContrastOptimizer = 1203;
			const int CodeToneMapToneBalancer = 1204;
			
			const int CodeBlendingAverage=1100;
			const int CodeBlendingAuto=1101;
			const int CodeBlendingAdjust=1102;
			const int CodeBlendingIntensive=1104;
			const int CodeBlendingRealistic=1105;
			
			const int CodeAtLeastOneOutputOK=5000;
			
			int method = 0;
			switch(code)
			{
				case CodeProcessRaw: msg = LocalText::GetTextTXTRAW(); break;
				case CodeAlign: msg = LocalText::GetTextMSGBATCHALIGN(); break;
				case CodeNoiseReduction: msg = LocalText::GetTextMsgBatchReducingNoise(); break;
				case CodeGenerateHdr: msg = LocalText::GetTextMSGBATCHHDRI(); break;
				case CodeToneMapDE: method = 1; msg = "Processing with Details Enhancer ...\n"; break;
				case CodeToneMapTC: method = 2; msg = "Processing with Tone Compressor ...\n"; break;
				case CodeToneMapContrastOptimizer: method = 3; msg = "Processing with Contrast Optimizer ...\n"; break;
				case CodeToneMapToneBalancer: method = 4; msg = "Processing with Tone Balancer ...\n"; break;
				case CodeBlendingAdjust: method = 10; msg = "Processing with Fusion/Natural ...\n"; break;
			}
			
			if(msg.empty())
				return true;
			
			if(method == 0)
			{
				string tmp, tmp2;
				fastformat::fmt(tmp, "{0}\n", msg);
				if(code != CodeNoiseReduction)
					fastformat::fmt(tmp2, tmp, std::to_string(this->bracketSize), this->firstFile);
				else
					tmp2 = tmp;
				
				msg = tmp2;
			}
			else
			{
			}
		}
		else {
			if(buf.find("Right after") != string::npos)
				return true;
			msg = buf;
		}
		
		
		//ch_stdout->read_to_end(buf) would never
		//return for an unknown reason
		this->textBuffer->insert_at_cursor(msg);
		
		this->batchForm->ScrollLogToEnd();
		
		this->batchForm->SetProgress((double)(++doneSteps) / this->totalSteps);
		
//		std::cout<<buf;
//		std::cout.flush();
		return true;
	}
	
	void BatchProcessor::Process()
	{
		// Configure environment
		std::vector <string> env;
		if (getenv("LD_LIBRARY_PATH"))
		{
			std::string e = string("LD_LIBRARY_PATH=") + getenv("LD_LIBRARY_PATH");
			env.push_back(e);
		}
		
		/*struct passwd* pw = getpwuid(getuid());
			const char* homedir = pw->pw_dir;
			std::str*/
		
		
		
		for(int i = 0; i < this->totalNumberOfSets; ++i)
		{
			// Configure and run command
			vector <string> setCommand = command;

			// Pass in the argument to PhotomatixCL that tells it that it's run from Photomatix
			if(AppProperties::IsRegistered()) {
			  unsigned val = bk3(gethostid());
			  char zOpt[9];
			  snprintf(zOpt, sizeof(zOpt), "%x", val);
			  setCommand.push_back("-Z");
			  setCommand.push_back(zOpt);
			}
			
			// Set number
			setCommand.push_back("-q");
			setCommand.push_back(std::to_string(i + 1));
			
			for(int j = 0; j < this->bracketSize; ++j)
			{
				setCommand.push_back(fileNames[i * this->bracketSize + j]);
			}
			this->firstFile = fileNames[i * this->bracketSize];
			int index = this->firstFile.find_last_of('/');
			this->firstFile = this->firstFile.substr(index + 1, this->firstFile.length() - index - 1);
			
			int std_out_desc = 0;
			
			string cmd;
			
			for(int j = 0; j < setCommand.size(); ++j)
			{
				cmd = cmd + setCommand[j];
				cmd = cmd + " ";
			}
			cout << cmd << endl;
			
			// Run command
			int pid = 0;
			int status = 0;
			Glib::spawn_async_with_pipes("", setCommand, env, Glib::SPAWN_CHILD_INHERITS_STDIN | Glib::SPAWN_DO_NOT_REAP_CHILD, sigc::slot< void >(), &pid, 0, &std_out_desc, 0);
			
			this->ch_stdout = Glib::IOChannel::create_from_fd(std_out_desc);
			sigc::connection connection = Glib::signal_io().connect(sigc::mem_fun(this, &BatchProcessor::TreatBatchOutput), std_out_desc, Glib::IO_IN);
			
			wait(0);
			//waitpid(pid, &status, WUNTRACED);
			
			//this->batchForm->SetProgress((double)(i + 1) / this->totalNumberOfSets);
			
			connection.disconnect();
			
			//this->ch_stdout->close(false);
			//this->ch_stdout.reset();
			
			this->textBuffer->insert_at_cursor("---------\n");
			//std::cout.flush();
		}
	}
	
	
	void BatchForm::SetProgress(double progress)
	{
		this->prgGlobal->set_fraction(progress);
	}
	
	BatchForm::BatchForm(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder):
		Gtk::Window(cobject)
	{
		this->activeFilesCount = 0;
		this->totalNumberOfSets = 0;
		this->bracketSize = 0;
		
		this->deSettings = NULL;
		this->efSettings = NULL;
		this->tcSettings = NULL;
		this->tbSettings = NULL;
		this->coSettings = NULL;
		
		this->processor = NULL;
		
		this->set_resizable(false);

		builder->get_widget("prgGlobal", this->prgGlobal);
	  builder->get_widget("btnCancel", this->btnCancel);
	  builder->get_widget("btnDestChoose", this->btnDestChoose);
	  builder->get_widget("btnRemove", this->btnRemove);
	  builder->get_widget("btnRun", this->btnRun);
	  builder->get_widget("btnSelect", this->btnSelect);
	  builder->get_widget("btnStop", this->btnStop);
	  builder->get_widget("btnTM1", this->btnTM1);
	  builder->get_widget("btnTM2", this->btnTM2);
	  builder->get_widget("btnTM3", this->btnTM3);
	  builder->get_widget("btnToneBalancer", this->btnToneBalancer);
	  builder->get_widget("btnAdjust", this->btnAdjust);
	  builder->get_widget("cbxHdrSavedAs", this->cbxHdrSavedAs);
	  builder->get_widget("cbxSaveAs", this->cbxSaveAs);
	  builder->get_widget("chkAdjust", this->chkAdjust);
	  builder->get_widget("chkAlign", this->chkAlign);
	  builder->get_widget("chkCrop", this->chkCrop);
	  builder->get_widget("chkPerspectiveCorrection", this->chkPerspectiveCorrection);
	  builder->get_widget("chkRemoveHDR", this->chkRemoveHDR);
	  builder->get_widget("chkTM1", this->chkTM1);
	  builder->get_widget("chkTM2", this->chkTM2);
	  builder->get_widget("chkTM3", this->chkTM3);
	  builder->get_widget("chkToneBalancer", this->chkToneBalancer);
	  builder->get_widget("cmbBracket", this->cmbBracket);
	  builder->get_widget("cmbFilter", this->cmbFilter);
	  builder->get_widget("label1", this->label1);
	  builder->get_widget("label2", this->label2);
	  builder->get_widget("label3", this->label3);
	  builder->get_widget("label4", this->label4);
	  builder->get_widget("label5", this->label5);
	  builder->get_widget("label6", this->label6);
	  builder->get_widget("label7", this->label7);
	  builder->get_widget("label8", this->label8);
	  builder->get_widget("labelDest", this->labelDest);
	  builder->get_widget("lblNotice", this->lblNotice);
	  //builder->get_widget("rbnAdvanced", this->rbnAdvanced);
	  builder->get_widget("rbnAlignRotation", this->rbnAlignRotation);
	  builder->get_widget("rbnAlignTranslation", this->rbnAlignTranslation);
	  builder->get_widget("rbnDestCustom", this->rbnDestCustom);
	  builder->get_widget("rbnDestUnderSource", this->rbnDestUnderSource);
	  builder->get_widget("rbnSelectionFiles", this->rbnSelectionFiles);
	  builder->get_widget("rbnSelectionFolder", this->rbnSelectionFolder);
	  //builder->get_widget("rbnSelect", this->rbnSelect);
	  builder->get_widget("txtDest", this->txtDest);
	  builder->get_widget("nudJpgQuality", this->nudJpgQuality);
	  builder->get_widget("txtFileList", this->lstFileList);
	  builder->get_widget("txtBatchLog", this->txtBatchLog);
	  
	  //builder->get_widget("btnOk", this->btnOk);
	  //builder->get_widget("cbxColorProfile", this->cbxColorProfile);
	  //builder->get_widget("cbxEVSpacing", this->cbxEVSpacing);
	  builder->get_widget("chkReduceNoise", this->chkReduceNoise);
	  builder->get_widget("cbxReduceNoiseTarget", this->cbxReduceNoiseTarget);
	  //builder->get_widget("cbxWhiteBalance", this->cbxWhiteBalance);
	  //builder->get_widget("chkForceSpacing", this->chkForceSpacing);
	  builder->get_widget("chkRemoveGhosts", this->chkRemoveGhosts);
	  //builder->get_widget("rbnAlways", this->rbnAlways);
	  //builder->get_widget("rbnNoEV", this->rbnNoEV);
	  //builder->get_widget("txtWhiteBalanceTemperature", this->txtWhiteBalanceTemperature);
	  builder->get_widget("sldReduceNoiseStrength", this->sldReduceNoiseStrength);
	  builder->get_widget("sldDeghosting", this->sldDeghosting);
	  
	  this->txtBatchLog->set_editable(false);
	  this->txtDest->set_editable(false);
	  
	  this->rbnSelectionFiles->signal_toggled().connect(sigc::mem_fun(this, &BatchForm::OnSelectFilesCheckedChanged));

	  const char *toggleWidgets[] = {
	    "chkTM1", "chkTM2", "chkTM3", "chkToneBalancer", "chkAdjust",
		"chkAlign", "rbnAlignTranslation", "rbnAlignRotation",
	    "rbnDestUnderSource", "rbnDestCustom"
	  };
	  
	  this->EnableDisableControls();
	  
	  for (const char *name : toggleWidgets)
	  {
	    Gtk::CheckButton *btn;
	    builder->get_widget(name, btn);
	    btn->signal_toggled().connect(sigc::mem_fun(this, &BatchForm::EnableDisableControls));
	  }
	  
	  this->cmbFilter->signal_changed().connect(sigc::mem_fun(*this, &BatchForm::OnFilterSelectedIndexChanged));
	  
	  this->btnTM1->signal_clicked().connect(sigc::mem_fun(this, &BatchForm::OnBtnDetailsEnhancerSettingsClicked));
	  this->btnTM2->signal_clicked().connect(sigc::mem_fun(this, &BatchForm::OnBtnToneCompressorSettingsClicked));
	  this->btnTM3->signal_clicked().connect(sigc::mem_fun(this, &BatchForm::OnBtnContrastOptimizerSettingsClicked));
	  this->btnToneBalancer->signal_clicked().connect(sigc::mem_fun(this, &BatchForm::OnBtnToneBalancerSettingsClicked));
	  this->btnAdjust->signal_clicked().connect(sigc::mem_fun(this, &BatchForm::OnBtnExposureFusionSettingsClicked));
	  
	  this->btnRemove->signal_clicked().connect(sigc::mem_fun(this, &BatchForm::OnBtnRemoveClicked));
	  this->btnCancel->signal_clicked().connect( [this] { this->hide(); });
	  
	  this->btnDestChoose->signal_clicked().connect(sigc::mem_fun(this, &BatchForm::OnBtnDestChooseClicked));
	  
	  this->btnRun->signal_clicked().connect(sigc::mem_fun(this, &BatchForm::OnBtnRunClicked));
	  this->btnSelect->signal_clicked().connect(sigc::mem_fun(this, &BatchForm::OnBtnSelectClicked));
	  
	  this->cmbBracket->signal_changed().connect(sigc::mem_fun(this, &BatchForm::OnCmbBracketChanged));
	  this->cbxSaveAs->signal_changed().connect(sigc::mem_fun(this, &BatchForm::OnCbxFileFormatChanged));
	  
	  this->chkRemoveGhosts->signal_toggled().connect(sigc::mem_fun(this, &BatchForm::OnRemoveGhostsCheckedChanged));
	  this->chkReduceNoise->signal_toggled().connect(sigc::mem_fun(this, &BatchForm::OnReduceNoiseCheckedChanged));
	  
	  this->rbnDestCustom->signal_toggled().connect(sigc::mem_fun(this, &BatchForm::OnCustomPathCheckedChanged));
	  
	  // Setup sliders
	  this->sldReduceNoiseStrength->set_digits(0);
	  this->sldReduceNoiseStrength->set_range(50.0, 150.0);
	  this->sldReduceNoiseStrength->set_value(100.0);
	  
	  this->sldDeghosting->set_digits(0);
	  this->sldDeghosting->set_range(1.0, 100.0);
	  this->sldDeghosting->set_value(50.0);
	  
	  
	  //this->signal_unrealize().connect(sigc::mem_fun(this, &BatchForm::OnClose));
	  
	  // Create source files list model
	  this->sourceFileList = Gtk::ListStore::create(this->sourceFileColumns);
	  this->lstFileList->set_model(this->sourceFileList);
	  this->lstFileList->append_column("File", this->sourceFileColumns.shortName);
	  this->lstFileList->get_selection()->set_mode(Gtk::SELECTION_MULTIPLE);
	  this->lstFileList->signal_cursor_changed().connect(sigc::mem_fun(this, &BatchForm::OnListSelectedItemsChanged));
	  
	  this->textBuffer = Gtk::TextBuffer::create();
	  this->txtBatchLog->set_buffer(this->textBuffer);
	  
	  
	  //this->signal_hide().connect( [this] { this->myRef.reset(); });
	  //this->signal_hide().connect( [this] { delete this; });
	  
	  ////////////////////////////////////////////
	  // Bind signals to controls
	  ////////////////////////////////////////////
	  
	  // General controls
	  #if 0
	  builder->get_widget("btnOK", button);
	  if (button)
		  button->signal_clicked().connect(sigc::mem_fun(*this, &BatchForm::OnBtnOkClicked)); 
	  
	  builder->get_widget("btnCancel", button);
	  if (button)
		  button->signal_clicked().connect(sigc::mem_fun(*this, &BatchForm::OnBtnCancelClicked));
	  
	  // Controls related to image alignment
	  builder->get_widget("chkAlign", this->chkAlignImages);
	  if (this->chkAlignImages)
		  this->chkAlignImages->signal_toggled().connect(sigc::mem_fun(*this, &BatchForm::OnAlignImagesCheckedChanged));
	  builder->get_widget("chkCrop", this->chkCrop);
	  builder->get_widget("rbnHorizVertShifts", this->rbnAlignShifts);
	  builder->get_widget("rbnMatchingFeatures", this->rbnAlignFeatures);
	  builder->get_widget("chkPerspective", this->chkPerspective);
	  builder->get_widget("lblPerspective", this->lblPerspective);
	  builder->get_widget("lblMaximumShift", this->lblMaximumShift);
	  builder->get_widget("lblMaximumShiftPercentage", this->lblMaximumShiftPercentage);
	  builder->get_widget("sldMaximumShift", this->sldMaximumShift);
	  
	  this->sldMaximumShift->set_value(12.0);
	  
	  
	  // Controls related to noise reduction
	  builder->get_widget("lblReduceNoiseStrength", this->lblReduceNoiseStrength);
	  builder->get_widget("lblReduceNoiseStrengthPercentage", this->lblReduceNoiseStrengthPercentage);
	  builder->get_widget("cbxReduceNoise", this->cbxReduceNoiseType);
	  
	  builder->get_widget("sldReduceNoiseStrength", this->sldReduceNoiseStrength);
	  this->sldReduceNoiseStrength->set_range(50.0, 150.0);
	  this->sldReduceNoiseStrength->set_value(100.0);
	  
	  builder->get_widget("chkReduceNoise", this->chkReduceNoise);
	  if (this->chkReduceNoise)
		  this->chkReduceNoise->signal_toggled().connect(sigc::mem_fun(*this, &BatchForm::OnReduceNoiseCheckedChanged));
	  
	  // Other controls
	  builder->get_widget("chkReduceCA", this->chkReduceChromaticAberrations);
	  
	  // Set initial enability of controls
	  this->OnAlignImagesCheckedChanged();
	  this->OnReduceNoiseCheckedChanged();
#endif

		if(!this->deSettings) {
			PhotomatixUtils::get_widget_derived("batch-settings-window", this->deSettings);
			this->deSettings->InitForm(1);
		}

		if(!this->tcSettings) {
			PhotomatixUtils::get_widget_derived("batch-settings-window", this->tcSettings);
			this->tcSettings->InitForm(2);
		}

		if(!this->coSettings) {
			PhotomatixUtils::get_widget_derived("batch-settings-window", this->coSettings);
			this->coSettings->InitForm(3);
		}
		
		if(!this->tbSettings) {
			PhotomatixUtils::get_widget_derived("batch-settings-window", this->tbSettings);
			this->tbSettings->InitForm(4);
		}

		if(!this->efSettings) {
			PhotomatixUtils::get_widget_derived("batch-settings-window", this->efSettings);
			this->efSettings->InitForm(10);
		}
		
		//PhotomatixUtils::get_widget_derived("more-preprocessing-options-window", this->moreOptions);

		this->ReadStickySettings();
		
		this->OnCmbBracketChanged();
	}
	
	BatchForm::~BatchForm()
	{
		this->SaveStickySettings();
		cout << "BATCH GOING AWAY!" << endl;
	}
	
	void BatchForm::OnSelectFilesCheckedChanged()
	{
		if(this->rbnSelectionFiles->get_active())
		{
			this->btnSelect->set_label("Select Files...");
		}
		else
		{
			this->btnSelect->set_label("Select Folder...");
		}
	}
	
	void BatchForm::OnListSelectedItemsChanged()
	{
		this->btnRemove->set_sensitive(true);
	}
	
	void BatchForm::ScrollLogToEnd()
	{
		Gtk::TextBuffer::iterator iter = this->textBuffer->end();
		this->txtBatchLog->scroll_to(iter);
	}
	
	void BatchForm::OnCustomPathCheckedChanged()
	{		
		this->txtDest->set_sensitive(this->rbnDestCustom->get_active());
		this->btnDestChoose->set_sensitive(this->rbnDestCustom->get_active());
	}

	void BatchForm::OnBtnDestChooseClicked()
	{
		Gtk::FileChooserDialog openFileDialog("Open", Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
		openFileDialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		openFileDialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);
		int result = openFileDialog.run();
		if(result == Gtk::RESPONSE_OK)
		{
			this->txtDest->set_text(openFileDialog.get_filename());
		}
	}

	void BatchForm::OnCbxFileFormatChanged()
	{
		if(this->cbxSaveAs->get_active_row_number() == 0) {
			this->nudJpgQuality->set_sensitive(true);
		} else {
			this->nudJpgQuality->set_sensitive(false);
		}
	}
	
	void BatchForm::OnRemoveGhostsCheckedChanged()
	{
		this->sldDeghosting->set_sensitive(this->chkRemoveGhosts->get_active());
	}
	
	void BatchForm::OnReduceNoiseCheckedChanged()
	{
		this->cbxReduceNoiseTarget->set_sensitive(this->chkReduceNoise->get_active());
		this->sldReduceNoiseStrength->set_sensitive(this->chkReduceNoise->get_active());
	}
	
	void BatchForm::OnFilterSelectedIndexChanged()
	{
		this->UpdateFilesList();
	}
	
	void BatchForm::OnBtnRemoveClicked()
	{
		std::vector <Gtk::TreeModel::Path> selectedRows = this->lstFileList->get_selection()->get_selected_rows();
		
		for(gint i = 0; i < selectedRows.size(); i++)
		{
			
			Gtk::TreeModel::iterator iter = this->lstFileList->get_model()->get_iter(selectedRows[i]);
			
			Gtk::TreeModel::Row row = *iter;
			int index = -1;
			for(int j = 0; j < this->allSourceFiles.size(); ++j)
			{
				if(this->allSourceFiles[j].fullPath == row[this->sourceFileColumns.fullPath])
					index = j;
			}
			
			if(index != -1) this->allSourceFiles.erase(this->allSourceFiles.begin() + index);
			// now do what you need to do with the data in your TreeModel
		}
		//vector <string> filenames;
		//for(auto& row : rows)
		//{
			//Glib::ustring file = row[this->sourceFileColumns.fullPath];
			//filenames.push_back(file);
		//}
		//this->sourceFileList->erase(this->sourceFileList->begin() + 
		this->UpdateFilesList();
		
		this->btnRemove->set_sensitive(false);
	}
	
	void BatchForm::UpdateFilesList()
	{
		this->activeFilesCount = 0;
		this->sourceFileList->clear();
		
		std::sort(allSourceFiles.begin(), allSourceFiles.end());
		for(int i = 0; i < allSourceFiles.size(); ++i)
		{	
			EImageFormatType fileFormat = PhotoImage::GetProbableFileFormat(allSourceFiles[i].shortName);
			if((fileFormat != EImageFormatType::Jpeg && this->cmbFilter->get_active_row_number() == 1) ||
			   (fileFormat != EImageFormatType::Tiff && this->cmbFilter->get_active_row_number() == 2) ||
			   (fileFormat != EImageFormatType::Raw && this->cmbFilter->get_active_row_number() == 3))
			{
				continue;
			}
			
			Gtk::TreeModel::Row row = *(this->sourceFileList->append());
			
			row[this->sourceFileColumns.shortName] = allSourceFiles[i].shortName;
			row[this->sourceFileColumns.fullPath] = allSourceFiles[i].fullPath;
			
			++this->activeFilesCount;
		}
	}
	
	void BatchForm::SaveStickySettings()
	{
		AppProperties::SetBatchBracketsNumber(this->cmbBracket->get_active_row_number());
		
		AppProperties::SetBatchAlignEnabled(this->chkAlign->get_active());
		AppProperties::SetBatchAlignTripod(this->rbnAlignTranslation->get_active());
		AppProperties::SetBatchAlignCrop(this->chkCrop->get_active());
		AppProperties::SetBatchAlignPerspectiveCorrection(this->chkPerspectiveCorrection->get_active());
		AppProperties::SetBatchFileFormat(this->cbxSaveAs->get_active_row_number());
		AppProperties::SetBatchRemoveHdr(this->chkRemoveHDR->get_active());
		
		AppProperties::SetBatchRemoveGhosts(this->chkRemoveGhosts->get_active());
		AppProperties::SetBatchReduceNoise(this->chkReduceNoise->get_active());
		AppProperties::SetBatchReduceNoiseType(this->cbxReduceNoiseTarget->get_active_row_number());
		AppProperties::SetBatchReduceNoiseStrength(this->sldReduceNoiseStrength->get_value());
		
		AppProperties::SetBatchSettingsTm1(this->chkTM1->get_active());
		AppProperties::SetBatchSettingsTm2(this->chkTM2->get_active());
		AppProperties::SetBatchSettingsTm3(this->chkTM3->get_active());
		AppProperties::SetBatchSettingsToneBalancer(this->chkToneBalancer->get_active());
		AppProperties::SetBatchSettingsFusion(this->chkAdjust->get_active());
		
		AppProperties::SetSaveJpegQuality(this->nudJpgQuality->get_value());
		//AppProperties::SetBatchColorSpace(this->cbxColorProfile->get_active_row_number());
		//AppProperties::SetBatchWhiteBalance(this->cbxWhiteBalance->get_active_row_number());
	}

	void BatchForm::ReadStickySettings()
	{
		this->cmbBracket->set_active(AppProperties::GetBatchBracketsNumber());
		
		this->chkAlign->set_active(AppProperties::GetBatchAlignEnabled());
		this->rbnAlignTranslation->set_active(AppProperties::GetBatchAlignTripod());
		this->chkCrop->set_active(AppProperties::GetBatchAlignCrop());
		this->chkPerspectiveCorrection->set_active(AppProperties::GetBatchAlignPerspectiveCorrection());
		this->cbxSaveAs->set_active(AppProperties::GetBatchFileFormat());
		this->chkRemoveHDR->set_active(AppProperties::GetBatchRemoveHdr());
		
		this->chkRemoveGhosts->set_active(AppProperties::GetBatchRemoveGhosts());
		this->chkReduceNoise->set_active(AppProperties::GetBatchReduceNoise());
		this->cbxReduceNoiseTarget->set_active(AppProperties::GetBatchReduceNoiseType());
		this->sldReduceNoiseStrength->set_value(AppProperties::GetBatchReduceNoiseStrength());
		
		this->chkTM1->set_active(AppProperties::GetBatchSettingsTm1());
		this->chkTM2->set_active(AppProperties::GetBatchSettingsTm2());
		this->chkTM3->set_active(AppProperties::GetBatchSettingsTm3());
		this->chkToneBalancer->set_active(AppProperties::GetBatchSettingsToneBalancer());
		this->chkAdjust->set_active(AppProperties::GetBatchSettingsFusion());
		
		this->nudJpgQuality->set_value(AppProperties::GetSaveJpegQuality());
		
		//this->cbxColorProfile->set_active(AppProperties::GetBatchColorSpace());
		//this->cbxWhiteBalance->set_active(AppProperties::GetBatchWhiteBalance());
		
		string lastPath = AppProperties::GetBatchLastOpenedDir();
		if(!lastPath.empty() && Glib::file_test(lastPath, Glib::FILE_TEST_EXISTS) && Glib::file_test(lastPath, Glib::FILE_TEST_IS_DIR))
		{
			this->OpenPath(lastPath);
		}
	}
	
	void BatchForm::OnBtnDetailsEnhancerSettingsClicked()
	{
		this->deSettings->present();
	}

	void BatchForm::OnBtnToneCompressorSettingsClicked()
	{
		this->tcSettings->present();
	}
	
	void BatchForm::OnBtnToneBalancerSettingsClicked()
	{
		this->tbSettings->present();
	}
	
	void BatchForm::OnCmbBracketChanged()
	{
		// TODO: localize text below
		Gtk::TreeModel::iterator iter = this->cmbBracket->get_active();
		if(iter) {
			Gtk::TreeModel::Row row = *iter;
			string value;
			row->get_value(1, value);
			
			string text;
			fastformat::fmt(text, "Batch will merge {0} files at a time, taking them from the files \nlisted in the Source frame below.", value);
			this->lblNotice->set_text(text);
		}
	}
		
	void BatchForm::OnBtnContrastOptimizerSettingsClicked()
	{
		this->coSettings->present();
	}

	void BatchForm::OnBtnExposureFusionSettingsClicked()
	{
		this->efSettings->present();
	}
	
	bool BatchForm::TreatBatchOutput(Glib::IOCondition cond)
	{
		/*Glib::ustring buf;
		
		this->ch_stdout->read_line(buf);
		//ch_stdout->read_to_end(buf) would never
		//return for an unknown reason
		this->textBuffer->insert_at_cursor(buf);
		//std::cout<<buf;
		return true;*/
	}
	
	std::string BatchForm::FindNextAvailableSubfolder(string& folder, string& prefix) const
	{
		int increment = 0; 
		bool dirExist = true;
		string path = folder + "/" + prefix;
		string strVal = path;
		while (dirExist)
		{
			++increment;
			
			ostringstream strIncrement;
			strIncrement << std::setfill('0') << std::setw(2) << increment;
			strVal = path + strIncrement.str();
			if(!Glib::file_test(strVal, Glib::FILE_TEST_EXISTS) || !Glib::file_test(strVal, Glib::FILE_TEST_IS_DIR))
				dirExist = false;
			//if (!new DirectoryInfo(strVal).Exists) dirExist = false;
		}
		return strVal;
	}
	
	void BatchForm::SetBatchFileNamesFromForm()
	{
		this->bracketSize = cmbBracket->get_active_row_number() + 1;
		bool all = this->bracketSize == 1;
		
		Gtk::TreeModel::Children rows = this->sourceFileList->children();
		vector <string> filenames;
		for(auto& row : rows)
		{
			Glib::ustring file = row[this->sourceFileColumns.fullPath];
			filenames.push_back(file);
		}
		
		// TODO: handle case when multipleSourceFolders is true, i.e. when process sub-folders option is used
		bool multipleSourceFolders = false;
		if(multipleSourceFolders) {
		} else {
			if(this->bracketSize == 1) this->totalNumberOfSets = 1;
			else this->totalNumberOfSets = filenames.size() / bracketSize;
			
			/*for (int iSet=0; iSet<totalNumberOfSets; iSet++) batchProcessor.NumberOfImagesInSet[iSet]=imagesInBracket;
			 * 
			 i *f (this.PartialSelect)
			 {
				 // Need to go through the files manually and put in the right ones
				 int maxBracketedFiles=totalNumberOfSets*nominalBracketSize, cntFiles=0;
				 ArrayList tempInputImages=new ArrayList();
				 foreach (string strPath in this.filteredPathList)
				 {
					 if (cntFiles<maxBracketedFiles) 
					 {
						 int imagePosInSet = cntFiles % nominalBracketSize;
						 // Test whether we should add the image, if partial selection is turned on
						 if (selectionMask[imagePosInSet])
							 tempInputImages.Add(strPath);
						 cntFiles++;
		}
		}
		batchProcessor.InputImages=(string[])tempInputImages.ToArray(typeof(string));
		}
		else
		{
			// Just copy the filtered path list into the batch processor directly
			batchProcessor.InputImages=(string[])this.filteredPathList.ToArray(typeof(string));
		}*/
		}
	}
	
	// TODO: run this below code on another thread?
	// TODO: check why logging doesn't always work correctly (some messages are apparently missing). Probably
	// it has something to do with multithreading/buffering
	void BatchForm::OnBtnRunClicked()
	{
		// No files were selected, so skip processing
		if(this->activeFilesCount == 0)
			return;
		
		this->btnRun->set_sensitive(false);
		
		// TODO: handle case when number of brackets is detected automatically
		bool autoBracket = false;
		if(autoBracket) {
			// ...
		} else {
			this->SetBatchFileNamesFromForm();
		}
		
		
		Gtk::TreeModel::Children rows = this->sourceFileList->children();
		vector <string> filenames;
		for(auto& row : rows)
		{
			Glib::ustring file = row[this->sourceFileColumns.fullPath];
			filenames.push_back(file);
		}
		
		
		int totalSteps = 0;
		
		
		// Configure command
		std::vector <string> command;
		command.push_back("/bin/bash");
#ifdef Release
		FILE* pipe = popen("whereis PhotomatixCL", "r");
		std::string result = "";
		if(pipe)
		{
			char buffer[128];
			while(!feof(pipe)) {
				if(fgets(buffer, 128, pipe) != NULL)
					result += buffer;
			}
			pclose(pipe);
			std::vector<std::string> strs = PhotomatixUtils::split(result, ' ');
			if(strs.size() > 1) result = strs[1].substr(0, strs[1].find("CL") + 2);
			else result = "";
			std::cout << result << std::endl;
		}
		// PhotomatixCL wasn't found so return
		if(result.empty()) return;
		
		command.push_back(result);
#else
		command.push_back("./PhotomatixCL");
#endif
		command.push_back("-z");

		if(this->chkTM1->get_active() || this->chkTM2->get_active() || this->chkTM3->get_active() || this->chkToneBalancer->get_active())
			++totalSteps;
		
		// Prepare settings files by writing settings to disk
		if(this->chkTM1->get_active()) {
			XmpSettings* settings = this->deSettings->GetCurrentSettings();

			string tmpFileName = tempnam("/tmp/", "pmp-");
			this->tmpFiles.push_back(tmpFileName);
			
			write_xmp(tmpFileName.c_str(), settings->method, settings->parameters);
			command.push_back("-x");
			command.push_back(tmpFileName);
			
			++totalSteps;
		}
		if(this->chkTM2->get_active()) {
			XmpSettings* settings = this->tcSettings->GetCurrentSettings();

			string tmpFileName = tempnam("/tmp/", "pmp-");
			this->tmpFiles.push_back(tmpFileName);
			
			write_xmp(tmpFileName.c_str(), settings->method, settings->parameters);
			command.push_back("-x");
			command.push_back(tmpFileName);
			
			++totalSteps;
		}
		if(this->chkTM3->get_active()) {
			XmpSettings* settings = this->coSettings->GetCurrentSettings();

			string tmpFileName = tempnam("/tmp/", "pmp-");
			this->tmpFiles.push_back(tmpFileName);
			
			write_xmp(tmpFileName.c_str(), settings->method, settings->parameters);
			command.push_back("-x");
			command.push_back(tmpFileName);
			
			++totalSteps;
		}
		if(this->chkToneBalancer->get_active()) {
			XmpSettings* settings = this->tbSettings->GetCurrentSettings();

			string tmpFileName = tempnam("/tmp/", "pmp-");
			this->tmpFiles.push_back(tmpFileName);
			
			write_xmp(tmpFileName.c_str(), settings->method, settings->parameters);
			command.push_back("-x");
			command.push_back(tmpFileName);
			
			++totalSteps;
		}
		if(this->chkAdjust->get_active()) {
			XmpSettings* settings = this->efSettings->GetCurrentSettings();

			string tmpFileName = tempnam("/tmp/", "pmp-");
			this->tmpFiles.push_back(tmpFileName);
			
			write_xmp(tmpFileName.c_str(), settings->method, settings->parameters);
			command.push_back("-x");
			command.push_back(tmpFileName);
			
			++totalSteps;
		}
		
		// Output file format
		command.push_back("-s");
		switch(this->cbxSaveAs->get_active_row_number())
		{
			case 0:
				command.push_back("jpg");
				command.push_back("-j");
				command.push_back(std::to_string(this->nudJpgQuality->get_value()));
				break;
			case 1:
				command.push_back("tif");
				command.push_back("-bi");
				command.push_back("8");
				break;
			case 2:
				command.push_back("tif");
				command.push_back("-bi");
				command.push_back("16");
				break;
		}
		
		// Noise reduction
		if(this->GetReduceNoise())
		{
			switch(this->GetNoiseReductionType())
			{
				case NoiseReductionUnderExposed: command.push_back("-no0"); break;
				case NoiseReductionNormalAndUnderExposed: command.push_back("-no1"); break;
				case NoiseReductionAll: command.push_back("-no2"); break;
			}
			
			command.push_back("-nr");
			command.push_back(std::to_string(this->GetNoiseReductionStrength()));
			
			++totalSteps;
		}
		
		// Deghosting
		if(this->GetRemoveGhosts())
		{
			command.push_back("-g");
			command.push_back(std::to_string(this->GetDeghostingStrength()));
		}
		
		// Alignment
		if(this->chkAlign->get_active()) {
			// Taken on Tripod
			if(this->rbnAlignTranslation->get_active()) {
				// Crop
				if(this->chkCrop->get_active()) {
					command.push_back("-a1");
				} else {
					command.push_back("-a1n");
				}
			}
			// Hand-held (WITH perspective correction)
			else if(this->rbnAlignRotation->get_active() && this->chkPerspectiveCorrection->get_active()) {
				// Crop
				if(this->chkCrop->get_active()) {
					command.push_back("-a2");
				} else {
					command.push_back("-a2n");
				}
			}
			// Hand-held (WITHOUT perspective correction)
			else if(this->rbnAlignRotation->get_active() && !this->chkPerspectiveCorrection->get_active()) {
				// Crop
				if(this->chkCrop->get_active()) {
					command.push_back("-a3");
				} else {
					command.push_back("-a3n");
				}
			}
			
			++totalSteps;
		}
		
		// Remove intermediary HDR image
		if(this->chkRemoveHDR->get_active()) {
			command.push_back("-h");
			command.push_back("remove");
		}
		
		// Miscellaneous settings
		// TODO: what to do with these settings?
		/*command.push_back("-co");
		command.push_back(std::to_string(this->moreOptions->GetColorSpace()));
		
		command.push_back("-wb");
		command.push_back(std::to_string(this->moreOptions->GetWhiteBalanceTemperature()));*/
		
		command.push_back("-mp");
		command.push_back(std::to_string(AppProperties::GetNumberOfProcessors()));
		
		command.push_back("-3");
		
		//command.push_back("-no2");
		// TODO: temporary name - use "real" one
		//command.push_back("/home/wojtek/Pictures/landscape/SampleLandscape/");
		{
			if (this->rbnDestCustom->get_active())
			{
				// Destination is not under source - we use custom path supplied
				outputPath = this->txtDest->get_text();
				outputPath.append("/");
			}
			else
			{
				// Destination is under source
				// TODO: handle this case!
				/*if (this.MultipleOutputFolders)
				{
					// We check all the folders in dirs, and find the highest folder number which is free in all
					// of them, and use that
					
					// First put all subfolders of the source folder into the list.
					ArrayList dirs = new ArrayList();
					DirectoryInfo [] subdirs = new DirectoryInfo(this.sourcePath).GetDirectories();
					foreach (DirectoryInfo subdir in subdirs)
						dirs.Add(subdir.FullName);
					
					// Next, find the highest free folder number
						int maxFolderNo = 0;
						foreach (string subdirPath in dirs)
						{
							int folderNo = FindNextAvailableSubfolder(subdirPath, LocalText.TextFileBatchResfolder);
							if (folderNo > maxFolderNo)
								maxFolderNo = folderNo;
						}
						
						// This will be the folder name everywhere
						string universalFolderName = LocalText.TextFileBatchResfolder + String.Format("{0:D2}", maxFolderNo);
						
						// Now make the folders
						try 
						{
							
							foreach (string subdirPath in dirs)
							{
								string folderName = subdirPath + "\\" + universalFolderName;
								Directory.CreateDirectory(folderName);
							}
						}
						catch (SystemException ex)	
						{
							Console.WriteLine("Creation PhotomatixResult directories failed: {0}", ex.ToString());
							throw;
							// TODO: We should put up a dialog box here, and abort.
						}
						
						// And set up the batch settings
						this.batchProcessor.OutputRelativeToSource = true;
						// When output is relative to source, we just store the subfolder name in the output path
						this.batchProcessor.OutputPath = universalFolderName;
				}
				else*/
				{
					// Not separate subfolders - just putting results into one folder
					outputPath = this->FindNextAvailableSubfolder(this->sourcePath, LocalText::GetTextFILEBATCHRESFOLDER());
					outputPath.append("/");
					//string chemin = this.sourcePath + "\\" + LocalText.TextFileBatchResfolder;
					//string strVal = chemin + string.Format("{0:D2}", increment);
					
					mkdir(outputPath.c_str(), 0700);
					//this.destPath=strVal;					
					//this.batchProcessor.OutputPath=this.destPath;				
				}
			}
		}
		
		command.push_back("-d");
		command.push_back(outputPath);
		
		this->totalNumberOfSets = this->activeFilesCount / bracketSize;
		
		// TODO: make processor object a class field and delete it after it finishes processing
		this->processor = shared_ptr<BatchProcessor> (new BatchProcessor(this->bracketSize == 1 ? 1 : this->totalNumberOfSets, this->bracketSize == 1 ? this->activeFilesCount : this->bracketSize, command, filenames, this->textBuffer, this, totalSteps));
		this->processor->sig_done.connect(sigc::mem_fun(*this, &BatchForm::OnBatchFinished));
		this->processor->Start();
		//delete processor;
		
		/*for(int i = 0; i < this->totalNumberOfSets; ++i)
		{
			// Configure and run command
			vector <string> setCommand = command;
			
			for(int j = 0; j < this->bracketSize; ++j)
			{
				setCommand.push_back(filenames[i * 3 + j]);
			}
			
			int std_out_desc;
			
			// Run command
			int pid;
			int status;
			Glib::spawn_async_with_pipes("", setCommand, env, Glib::SPAWN_CHILD_INHERITS_STDIN, sigc::slot< void >(), &pid, 0, &std_out_desc, 0);
			waitpid(-1, &status, 0);
			//waitpid(pid);
			//string str;
			//Glib::spawn_sync("", command, env, Glib::SPAWN_CHILD_INHERITS_STDIN, sigc::slot< void >(), &str, 0, 0);
			
			this->ch_stdout = Glib::IOChannel::create_from_fd(std_out_desc);
			Glib::signal_io().connect(sigc::mem_fun(this, &BatchForm::TreatBatchOutput), std_out_desc, Glib::IO_IN);
		}*/
			
	}
	
	void BatchForm::OnBatchFinished()
	{
		// Remove temporary files
		for(int i = 0; i < this->tmpFiles.size(); ++i)
			std::remove(this->tmpFiles[i].c_str());
		
		string resultsPath = LocalText::GetTextMSGCONVEND4();//"Results are in:"
		resultsPath += ": \n" + this->outputPath + "\n";
		this->textBuffer->insert_at_cursor(resultsPath);
		this->ScrollLogToEnd();
		
		// Destruct current batch processor
		this->processor.reset();
		
		this->btnRun->set_sensitive(true);
		this->prgGlobal->set_fraction(0.0);
	}
	
	void BatchForm::OnBtnSelectClicked()
	{
		if(rbnSelectionFiles->get_active())
		{
			OpenImageForm openFileDialog;//("Open", Gtk::FILE_CHOOSER_ACTION_OPEN);
			//openFileDialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
			//openFileDialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);
			openFileDialog.set_select_multiple(true);
			int result = openFileDialog.run();
			if(result == Gtk::RESPONSE_OK)
			{
				this->sourceFileList->clear();
				this->allSourceFiles.clear();
			
				vector <string> selectedFiles = openFileDialog.get_filenames();
				
				bool sourcePathSet = false;
				
				for(int i = 0; i < selectedFiles.size(); ++i)
				{
					//Gtk::TreeModel::Row row = *(this->sourceFileList->append());
					
					string fileName = selectedFiles[i];
					int index = fileName.find_last_of('/');
					
					if(!sourcePathSet) {
						this->sourcePath = fileName.substr(0, index + 1);
						sourcePathSet = true;
						AppProperties::SetBatchLastOpenedDir(this->sourcePath);
					}
					
					fileName = fileName.substr(index + 1, fileName.length() - index - 1);
					
					FileListEntry entry {fileName, selectedFiles[i]};
					this->allSourceFiles.push_back(entry);
					
					//row[this->sourceFileColumns.shortName] = fileName;
					//row[this->sourceFileColumns.fullPath] = selectedFiles[i];
				}
			}
			
			this->UpdateFilesList();
		}
		else
		{
			Gtk::FileChooserDialog openFileDialog("Open", Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
			openFileDialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
			openFileDialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);
			int result = openFileDialog.run();
			if(result == Gtk::RESPONSE_OK)
			{
				string path = openFileDialog.get_filename();
				AppProperties::SetBatchLastOpenedDir(path);
				
				this->OpenPath(path);
				/*DirectoryInfo selectedDirectory = new DirectoryInfo(browserDialog.SelectedPath);
				this.sourcePath=selectedDirectory.FullName;
				if (selectedDirectory.FullName!=null) 
				{
					hasPaths=true;
					getAndSetPathListFromFolder(selectedDirectory);
					if (rbnDestUnderSource.Checked)
					{
						DisplayDestination(selectedDirectory.FullName);
					}
				}
				if (this.chkProcessSubfolders.Checked) updateDest(); */
			}
		}
	}
	
	void BatchForm::OpenPath(string& path)
	{
		this->sourceFileList->clear();
		this->allSourceFiles.clear();
		
		Glib::RefPtr<Gio::File> file = Gio::File::create_for_path(path);
		Glib::RefPtr<Gio::FileEnumerator> child_enumeration = file->enumerate_children(G_FILE_ATTRIBUTE_STANDARD_NAME);
		//std::vector<Glib::ustring> file_names;
		Glib::RefPtr<Gio::FileInfo> file_info;
		
		while ((file_info = child_enumeration->next_file()) != NULL)
		{
			string fullPath;
			string fileName = file_info->get_name();
			this->sourcePath = child_enumeration->get_container()->get_path();
			fastformat::fmt(fullPath, "{0}/{1}", this->sourcePath, fileName);
			EImageFormatType fileFormat = PhotoImage::GetProbableFileFormat(fileName);
			if(fileFormat != EImageFormatType::Unknown && fileFormat != EImageFormatType::Hdr)
			{
				FileListEntry entry {fileName, fullPath};
				this->allSourceFiles.push_back(entry);
			}
		}
		
		this->UpdateFilesList();
	}
	
	void BatchForm::EnableDisableControls()
	{
	  this->btnTM1->set_sensitive(GetTM1());
	  this->btnTM2->set_sensitive(GetTM2());
	  this->btnTM3->set_sensitive(GetTM3());
	  this->btnToneBalancer->set_sensitive(GetToneBalancer());
	  this->btnAdjust->set_sensitive(GetAdjust());
	  //this->cmbBracket->set_sensitive(GetSelect());
	  this->rbnAlignTranslation->set_sensitive(GetAlign());
	  this->rbnAlignRotation->set_sensitive(GetAlign());
	  this->chkCrop->set_sensitive(GetAlign());
	  this->chkPerspectiveCorrection->set_sensitive(GetAlign() && GetAlignRotation());
	  
	  this->btnDestChoose->set_sensitive(GetDestCustom());
	 
	}
	
#if 0	
	void BatchForm::OnBtnOkClicked()
	{
		this->hide();
		
		// TODO: move this to progress form or something similar		
		this->hdrGenerator->SetReduceChromaticAberrations(this->chkReduceChromaticAberrations->get_active());
		
		if(this->chkAlignImages->get_active())
		{
			this->hdrGenerator->SetAlignmentEnabled(true);
			PhotomatixImaging::EAlignmentType type = PhotomatixImaging::EAlignmentType::AlignmentNone;
			if(this->rbnAlignShifts->get_active())
				type = PhotomatixImaging::EAlignmentType::AlignmentTranslation;
			else if(this->chkPerspective->get_active())
			{
				type = PhotomatixImaging::EAlignmentType::AlignmentRotationPerspective;
				this->hdrGenerator->SetMaxShiftPerspective(this->sldMaximumShift->get_value());
			}
			else
				type = PhotomatixImaging::EAlignmentType::AlignmentRotation;
			
			this->hdrGenerator->SetAlignmentType(type);
			this->hdrGenerator->SetCropImagesEnabled(this->chkCrop->get_active());
		}
		else
			this->hdrGenerator->SetAlignmentEnabled(false);
		
		if(this->chkReduceNoise->get_active())
		{
			this->hdrGenerator->SetNoiseReductionType((PhotomatixImaging::ENoiseReductionType)this->cbxReduceNoiseType->get_active_row_number());
			this->hdrGenerator->SetNoiseReductionStrength(this->sldReduceNoiseStrength->get_value());
		}
		else
			this->hdrGenerator->SetNoiseReductionType(PhotomatixImaging::ENoiseReductionType::NoiseReductionNone);
		
		this->hdrGenerator->sig_done.connect(sigc::mem_fun(*this, &BatchForm::OnGeneratingHDRDone));
		this->hdrGenerator->Start();
	}
	
	void BatchForm::OnBtnCancelClicked()
	{
		this->hide();
	}
	
	void BatchForm::OnReduceNoiseCheckedChanged()
	{
		bool enabled = this->chkReduceNoise->get_active();
		this->cbxReduceNoiseType->set_sensitive(enabled);
		this->lblReduceNoiseStrength->set_sensitive(enabled);
		this->lblReduceNoiseStrengthPercentage->set_sensitive(enabled);
		this->sldReduceNoiseStrength->set_sensitive(enabled);
	}
	
	void BatchForm::OnGeneratingHDRDone()
	{
		DisplayableImageForm* form = DisplayableImageForm::OpenInstance();
		form->SetPhoto(hdrGenerator->GetOutputPhoto());
		//std::cout << "BBBB" << std::endl;
	}
	
	void BatchForm::OnAlignImagesCheckedChanged()
	{
		bool enabled = this->chkAlignImages->get_active();
		this->chkCrop->set_sensitive(enabled);
		this->rbnAlignShifts->set_sensitive(enabled);
		this->rbnAlignFeatures->set_sensitive(enabled);
		this->chkPerspective->set_sensitive(enabled);
		this->lblPerspective->set_sensitive(enabled);
		this->lblMaximumShift->set_sensitive(enabled);
		this->lblMaximumShiftPercentage->set_sensitive(enabled);
		this->sldMaximumShift->set_sensitive(enabled);
	}
#endif
}
