#include <iostream>
#include "displayableimageform.h"
#include "GladeUtils.h"
#include "load_bracketed_dialog.h"
#include "singlerawconversiondialog.h"
#include "openimageform.h"
#include "batchform.h"
#include "aboutform.h"

#include "hdrcontroller.h"
#include "processphotocontroller.h"
#include "draggedfilesdialog.h"
#include "../PhotomatixImaging/photoimage.h"
#include "../PhotomatixMisc/appproperties.h"
#include "localtext.h"
#include "../PhotomatixImaging/photoimage.h"


using namespace std;
using namespace PhotomatixControls;
using namespace PhotomatixImaging;
using namespace PhotomatixMisc;


namespace Photomatix
{
	int DisplayableImageForm::OpenedImagesCount = 0;
	DisplayableImageForm* DisplayableImageForm::MainForm = 0;

	
	DisplayableImageForm::DisplayableImageForm(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& gladeBuilder):
		Gtk::Window(cobject),
		unsavedChanges(false),
		cancelCloseEvent(false),
		displayableImageControl(0),
		builder(gladeBuilder),
		zoomFactor(1.0f)
	{
		this->set_resizable(true);
		this->set_default_size(400, 300);
		this->set_size_request(400, 300);
		if(AppProperties::GetLoadPhotosX() != -1 && AppProperties::GetLoadPhotosY() != -1) {
			this->move(AppProperties::GetLoadPhotosX(), AppProperties::GetLoadPhotosY());
		}
		else
			this->set_position(Gtk::WIN_POS_CENTER);
		this->set_title("Photomatix for Linux");
#ifdef Release
		this->set_icon_from_file("/usr/share/photomatix/icon.png");
#endif
		
		this->InitSignals();
		
		std::vector<Gtk::TargetEntry> listTargets;
		listTargets.push_back(Gtk::TargetEntry("text/uri-list"));
		drag_dest_set(listTargets, Gtk::DEST_DEFAULT_MOTION | Gtk::DEST_DEFAULT_DROP, Gdk::ACTION_COPY | Gdk::ACTION_MOVE);
		signal_drag_data_received().connect(sigc::mem_fun(*this, &DisplayableImageForm::OnFileDropped));
		
		if(DisplayableImageForm::OpenedImagesCount == 0)
			DisplayableImageForm::MainForm = this;
	}
	
	DisplayableImageForm::~DisplayableImageForm()
	{
	}
	
	DisplayableImageForm* DisplayableImageForm::CreateInstance()
	{
		DisplayableImageForm* window=0;
 		if(DisplayableImageForm::OpenedImagesCount == 0)
			window = DisplayableImageForm::MainForm;
		else
		{
			PhotomatixUtils::get_widget_derived("displayable-image-dialog", window);
			ProcessPhotoController* newController = new ProcessPhotoController();
			newController->SetImageWindow(window);
			window->signal_tonemapping_requested.connect(sigc::mem_fun(*newController, &ProcessPhotoController::OnToneMappingRequested));
			window->present();
		}
		
		++DisplayableImageForm::OpenedImagesCount;
		
		return window;
	}
	
	void DisplayableImageForm::InitSignals()
	{
		Gtk::MenuItem* menuItem;
		
		// Connect Signals
		builder->get_widget("mnuOpen", menuItem);
		if (menuItem)
			menuItem->signal_activate().connect(sigc::mem_fun(*this, &DisplayableImageForm::OnMnuOpenClicked));
		
		builder->get_widget("mnuLoadBracketed", menuItem);
		if (menuItem)
			menuItem->signal_activate().connect(sigc::mem_fun(*this, &DisplayableImageForm::OnMnuLoadBracketedClicked));
		
		builder->get_widget("mnuClose", menuItem);
		if (menuItem)
			menuItem->signal_activate().connect(sigc::mem_fun(*this, &DisplayableImageForm::OnMnuCloseClicked));
		
		/*builder->get_widget("mnuSave", menuItem);
		if (menuItem)
			menuItem->signal_activate().connect(sigc::mem_fun(*this, &DisplayableImageForm::OnMnuSaveClicked));
		
		builder->get_widget("mnuSaveAs", menuItem);
		if (menuItem)
			menuItem->signal_activate().connect(sigc::mem_fun(*this, &DisplayableImageForm::OnMnuSaveAsClicked));*/
		
		//builder->get_widget("mnuExposureDown", menuItem);
		//if (menuItem)
		//	menuItem->signal_activate().connect(sigc::mem_fun(*this, &DisplayableImageForm::OnMnuExposureDownClicked));
		
		//builder->get_widget("mnuExposureUp", menuItem);
		//if (menuItem)
		//	menuItem->signal_activate().connect(sigc::mem_fun(*this, &DisplayableImageForm::OnMnuExposureUpClicked));
		
		//builder->get_widget("mnuPreferences", menuItem);
		//if (menuItem)
//			menuItem->signal_activate().connect(sigc::mem_fun(*this, &DisplayableImageForm::OnMnuPreferencesClicked));
			
		builder->get_widget("mnuBatchBracketed", menuItem);
		if (menuItem)
			menuItem->signal_activate().connect(sigc::mem_fun(*this, &DisplayableImageForm::OnMnuBatchProcessingClicked));

		builder->get_widget("mnuToneMapping", menuItem);
		if (menuItem)
			menuItem->signal_activate().connect(sigc::mem_fun(*this, &DisplayableImageForm::OnMnuToneMappingClicked));
		
		builder->get_widget("mnuAbout", menuItem);
		if (menuItem)
			menuItem->signal_activate().connect(sigc::mem_fun(*this, &DisplayableImageForm::OnMnuAboutClicked));
		
		builder->get_widget("btnBrowse", this->btnBrowse);
		if (btnBrowse)
			btnBrowse->signal_clicked().connect(sigc::mem_fun(*this, &DisplayableImageForm::OnBrowseButtonClicked)); 
		
		builder->get_widget("instructions-box", this->instructionsBox);
	}
	
	void DisplayableImageForm::InitForm(bool move)
	{
		Gtk::Box* vbox=0;
		builder->get_widget("dialog-vbox", vbox);
		//this->remove();
		
		if(vbox)
		{	
			//this->add(*this->displayableImageControl);
			vbox->pack_start(*this->displayableImageControl);
			string fileName = this->displayableImageControl->GetPhoto()->GetFileName();
			int index = fileName.find_last_of("//");
			this->set_title(fileName.substr(index + 1));
			this->displayableImageControl->ShowScrollbars(false);
			this->displayableImageControl->signal_mouse_moved.connect(sigc::mem_fun(*this, &DisplayableImageForm::OnImageMouseMoved));
			this->displayableImageControl->signal_mouse_down.connect(sigc::mem_fun(*this, &DisplayableImageForm::OnImageMouseDown));
			this->displayableImageControl->signal_mouse_up.connect(sigc::mem_fun(*this, &DisplayableImageForm::OnImageMouseUp));
			
			this->SetOriginalZoom();
			
			if(move) {
				if(AppProperties::GetLoadPhotosX() != -1 && AppProperties::GetLoadPhotosY() != -1) {
					this->move(AppProperties::GetLoadPhotosX(), AppProperties::GetLoadPhotosY());
				}
				else
					this->set_position(Gtk::WIN_POS_CENTER);
			}
			this->set_resizable(false);
		}
		
		this->show_all_children();
	}
	
	void DisplayableImageForm::SetOriginalZoom()
	{
		int x, y;
		this->get_position(x, y);
		Glib::RefPtr <Gdk::Screen> screen = Gdk::Screen::get_default();
		int monitorIdx = screen->get_monitor_at_point(x, y);
		Gdk::Rectangle workArea = screen->get_monitor_workarea(monitorIdx);
		
		uint width2Thirds = workArea.get_width() * 2 / 3;
		uint height2Thirds = workArea.get_height() * 2 / 3;
		
		shared_ptr <PhotoImage> photo = this->displayableImageControl->GetPhoto();
		if(photo)
		{
			if (photo->GetWidth() > width2Thirds || photo->GetHeight() > height2Thirds)
			{
				if (photo->GetWidth() > photo->GetHeight())
				{
					this->zoomFactor = ((float)photo->GetWidth()) / width2Thirds;
					this->set_default_size(width2Thirds, (int)(photo->GetHeight() / zoomFactor));
				}
				else
				{
					this->zoomFactor = ((float)photo->GetHeight()) / height2Thirds;
					this->set_default_size((int)(photo->GetWidth() / zoomFactor), height2Thirds);
				}
			}
			else this->zoomFactor = 1.0f;
		}
		
		this->zoomFactor = 1.0f / this->zoomFactor;
		this->displayableImageControl->SetZoomFactor(this->zoomFactor);
	}
	
	void DisplayableImageForm::SetMenuItemsEnability(bool enabled)
	{
		Gtk::MenuItem* menuItem;
		
		builder->get_widget("mnuToneMapping", menuItem);
		if (menuItem) menuItem->set_sensitive(enabled);
		
		/*builder->get_widget("mnuSave", menuItem);
		if (menuItem) menuItem->set_sensitive(enabled);
		
		builder->get_widget("mnuSaveAs", menuItem);
		if (menuItem) menuItem->set_sensitive(enabled);*/
		
		// HDR view menus
		//builder->get_widget("mnuAdjustHDRView", menuItem);
		//if (menuItem) menuItem->set_sensitive(enabled && this->displayableImageControl->GetPhoto()->IsHDR());
		
		//builder->get_widget("mnuExposureUp", menuItem);
		//if (menuItem) menuItem->set_sensitive(enabled && this->displayableImageControl->GetPhoto()->IsHDR());
		
		//builder->get_widget("mnuExposureDown", menuItem);
		//if (menuItem) menuItem->set_sensitive(enabled && this->displayableImageControl->GetPhoto()->IsHDR());
	}
	
	void DisplayableImageForm::OnMnuToneMappingClicked()
	{
		this->signal_tonemapping_requested.emit();
	}
	
	void DisplayableImageForm::OnMnuLoadBracketedClicked()
	{
		shared_ptr <HdrController> hdrController = shared_ptr <HdrController>(new HdrController());
		hdrController->signal_hdr_generated.connect(sigc::mem_fun(*this, &DisplayableImageForm::OnHdrGenerated));
		hdrController->Init();
		
		DisplayableImageForm* form = DisplayableImageForm::CreateInstance();
		form->SetHdrController(hdrController);
		hdrController->SetImageWindow(form);
	}
	
	void DisplayableImageForm::OnMnuPreferencesClicked()
	{
		//PreferencesForm* preferencesform;
		//PhotomatixUtils::get_widget_derived("preferences-form", preferencesform);
		//preferencesform->present();
	}
	
	void DisplayableImageForm::OnMnuAboutClicked()
	{	
		AboutForm* aboutDialog = NULL;
		PhotomatixUtils::get_widget_derived("about-dialog", aboutDialog);
		aboutDialog->run();
		aboutDialog->hide();
	}
	
	void DisplayableImageForm::on_hide()
	{
		int x, y;
		this->get_position(x, y);
		AppProperties::SetLoadPhotosX(x);
		AppProperties::SetLoadPhotosY(y);
		
		Gtk::Widget::on_hide();
	}

	
	void DisplayableImageForm::OnMnuCloseClicked()
	{
		this->cancelCloseEvent = false;
		// TODO: unsavedChanges flag needs to be set somewhere to display message box. For this we need to implement utilities/tonemapping as otherwise there is nothing to save really
		if(this->unsavedChanges)
		{
			Gtk::MessageDialog dialog(*this, "Unsaved changes", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO);
			dialog.set_secondary_text(LocalText::GetTextMSGSAVE());
			dialog.add_button("Cancel", Gtk::RESPONSE_CANCEL);
			
			int result = dialog.run();
			if(result == Gtk::RESPONSE_CANCEL)
			{
				this->cancelCloseEvent = true;
				return;
			}
		}
		
		// We need to reset HdrController
		this->hdrController.reset();
		
		--DisplayableImageForm::OpenedImagesCount;
		if(DisplayableImageForm::OpenedImagesCount > 0)
			this->hide();
		else if(DisplayableImageForm::OpenedImagesCount == 0)
		{
			Gtk::Box* vbox=0;
			this->builder->get_widget("dialog-vbox", vbox);
			if(vbox)
				vbox->remove(*this->displayableImageControl);
			this->displayableImageControl.reset();
			
			this->set_resizable(true);
			this->resize(400, 300);
			if(AppProperties::GetLoadPhotosX() != -1 && AppProperties::GetLoadPhotosY() != -1) {
				this->move(AppProperties::GetLoadPhotosX(), AppProperties::GetLoadPhotosY());
			}
			else
				this->set_position(Gtk::WIN_POS_CENTER);
			this->set_title("Photomatix for Linux");
			
			this->SetMenuItemsEnability(false);
			
			DisplayableImageForm::MainForm = this;
			
			this->instructionsBox->set_visible(true);
		}
		else
		{
			Gtk::Main::instance()->quit();
		}
	}
	
	void DisplayableImageForm::OnMnuOpenClicked()
	{
		// Shows the window and returns when it is closed.
		OpenImageForm openImageDialog;
		
		int result = openImageDialog.run();
		
		//Handle the response:
		openImageDialog.hide();
		if(result == Gtk::RESPONSE_OK)
		{
			AppProperties::SetLastOpenDirectory(openImageDialog.get_current_folder());
			
			this->signal_file_open_requested.emit(openImageDialog.get_filename());
		}
	}
	
	void DisplayableImageForm::OnMnuSaveClicked()
	{
		// TODO: implement this. We need to have something to save
	}
	
	void DisplayableImageForm::OnMnuSaveAsClicked()
	{
		// TODO: implement this. We need to have something to save
	}
	
	bool DisplayableImageForm::OpenImage(const std::string& fileName)
	{
		if(!this->displayableImageControl)
			this->displayableImageControl = shared_ptr<ScrollableDisplayableImageControl>(new ScrollableDisplayableImageControl());
		
		int colorSpace = 0;
		int colorTemperature = 0;
		SingleRawConversionDialog* conversionDialog = NULL;
		if(PhotoImage::GetProbableFileFormat(fileName) == EImageFormatType::Raw) {
			PhotomatixUtils::get_widget_derived("single-raw-conversion-dialog", conversionDialog);
			int result = conversionDialog->run();
			conversionDialog->hide();
			if(result < 0)
				return false;
			
			colorSpace = conversionDialog->GetColorSpace();
			colorTemperature = conversionDialog->GetColorTemperature();
		}
		this->displayableImageControl->Open(fileName, false, colorTemperature, colorSpace);
		if(PhotoImage::GetProbableFileFormat(fileName) == EImageFormatType::Raw && conversionDialog->GetReduceNoise())
			this->displayableImageControl->GetPhoto()->ReduceNoise(conversionDialog->GetNoiseReductionStrength());
		if(PhotoImage::GetProbableFileFormat(fileName) != EImageFormatType::Raw && PhotoImage::GetProbableFileFormat(fileName) != EImageFormatType::Hdr)
			this->InitForm();
		this->SetMenuItemsEnability(true);
		
		this->instructionsBox->set_visible(false);
		
		return true;
	}
	
	bool DisplayableImageForm::on_delete_event(GdkEventAny* event)
	{
		this->OnMnuCloseClicked();
		
		if(DisplayableImageForm::OpenedImagesCount == 0 || this->cancelCloseEvent)
			return true;
		else
			return false;
	}
	
	void DisplayableImageForm::OnMnuExposureDownClicked()
	{
		if(this->displayableImageControl)
		{
			this->displayableImageControl->ExposureDown();
			this->SetOriginalZoom();
			this->show_all_children();
		}
	}
	
	void DisplayableImageForm::OnMnuExposureUpClicked()
	{
		if(this->displayableImageControl)
		{
			this->displayableImageControl->ExposureUp();
			this->SetOriginalZoom();
			this->show_all_children();
		}
	}

	void DisplayableImageForm::OnMnuBatchProcessingClicked()
	{
		auto form = BatchForm::GetInstance();
		form->present();
	}

	void DisplayableImageForm::OnFileDropped(const Glib::RefPtr<Gdk::DragContext>& context, int x, int y,
											 const Gtk::SelectionData& selection_data, guint info, guint time)
	{
		const int length = selection_data.get_length();
		if(length >= 0 && selection_data.get_format() == 8)
		{
			std::istringstream iss(selection_data.get_data_as_string());
			std::string uri;
			std::vector <string> files;
			
			while (std::getline(iss, uri))
			{
				if (uri.substr(0,8) == "file:///")
				{
					std::string full_path = Glib::filename_from_uri(uri);
					
					// chop off any newlines and carriage returns in the uri list
					while (full_path.c_str()[full_path.size()-1] == '\n' || full_path.c_str()[full_path.size()-1] == '\r')
						full_path.erase(full_path.size()-1);
					
					files.push_back(full_path);
				}
			}
			
			if(files.size() == 1)
			{
				if(PhotomatixImaging::PhotoImage::GetProbableFileFormat(files[0]) != PhotomatixImaging::EImageFormatType::Unknown)
				{
					this->signal_file_open_requested.emit(files[0]);
				}
			}
			else if(files.size() > 1)
			{
				bool merge = false;
				if(AppProperties::GetBypassDraggedFilesDialog() || AppProperties::GetBypassAllDialogsBeforeTm()) {
					merge = true;
				} else {
					DraggedFilesDialog* draggedFilesDialog = NULL;
					PhotomatixUtils::get_widget_derived("dragged-files-dialog", draggedFilesDialog);
					draggedFilesDialog->run();
					draggedFilesDialog->hide();
					
					merge = draggedFilesDialog->GetMergeToHdr();
				}
				
				if(merge)
				{
					shared_ptr <HdrController> hdrController = shared_ptr <HdrController>(new HdrController());
					hdrController->signal_hdr_generated.connect(sigc::mem_fun(*this, &DisplayableImageForm::OnHdrGenerated));
					hdrController->Init(files);
					
					DisplayableImageForm* form = DisplayableImageForm::CreateInstance();
					form->SetHdrController(hdrController);
					hdrController->SetImageWindow(form);
				}
				else
				{
					for(auto& file : files)
					{
						if(PhotomatixImaging::PhotoImage::GetProbableFileFormat(file) != PhotomatixImaging::EImageFormatType::Unknown)
						{
							this->signal_file_open_requested.emit(file);
						}
					}
				}
			}
		}
	}
	
	void DisplayableImageForm::SetPhoto(std::shared_ptr <PhotomatixImaging::PhotoImage> photo, bool init)
	{
		if(!this->displayableImageControl)
			this->displayableImageControl = shared_ptr<ScrollableDisplayableImageControl>(new ScrollableDisplayableImageControl());
		
		this->displayableImageControl->SetPhoto(photo);
		this->InitForm(init);
		this->SetMenuItemsEnability(true);
		
		this->instructionsBox->set_visible(false);
	}
	
	std::shared_ptr <PhotoImage> DisplayableImageForm::GetPhoto()
	{
		return this->displayableImageControl->GetPhoto();
	}
	
	void DisplayableImageForm::OnHdrGenerated()
	{
		//this->hdrController.reset();
	}
	
	void DisplayableImageForm::OnBrowseButtonClicked()
	{
		this->OnMnuLoadBracketedClicked();
	}
	
	void DisplayableImageForm::OnImageMouseMoved(GdkEventMotion* evt)
	{
		this->signal_mouse_moved.emit(evt);
	}
	
	void DisplayableImageForm::OnImageMouseDown(GdkEventButton* evt)
	{
		this->signal_mouse_down.emit(evt);
	}

	void DisplayableImageForm::OnImageMouseUp(GdkEventButton* evt)
	{
		this->signal_mouse_up.emit(evt);
	}
	
	void DisplayableImageForm::ShowCropRectangle(Gdk::Rectangle cropRect)
	{
		this->displayableImageControl->ShowCropRectangle(cropRect);
	}
}
