#ifndef DISPLAYABLEIMAGEFORM_H
#define DISPLAYABLEIMAGEFORM_H

#include <gtkmm.h>
#include <memory>
#include "displayableimagecontrol.h"
#include "scrollabledisplayableimagecontrol.h"


namespace PhotomatixImaging
{
	class PhotoImage;
}

namespace Photomatix
{
	class HdrController;
	
	/** Displays a single photo.
	 * @remarks
	 * This class serves purpose of main form. Application starts with an empty
	 * displayable image form to which user can load photos using main menu.
	 */
	class DisplayableImageForm : public Gtk::Window
	{
	private:
		/// Tells if there are any unsaved changes done to the image.
		bool unsavedChanges;
		bool cancelCloseEvent;
		
		Gtk::Button* btnBrowse;
		Gtk::Box* instructionsBox;
		
		
		void InitSignals();

	protected:
		/// Displayable image control used for displaying a photo
		std::shared_ptr <PhotomatixControls::ScrollableDisplayableImageControl> displayableImageControl;
		std::shared_ptr <HdrController> hdrController;
		
		/// Glade builder used to access widgets
		Glib::RefPtr<Gtk::Builder> builder;
		
		float zoomFactor;							///< Zoom factor used when displaying an image
		
		
		void SetMenuItemsEnability(bool enabled);
		
		//@{
		/** Signal Handlers */
		void OnMnuOpenClicked();
		void OnMnuLoadBracketedClicked();
		void OnMnuCloseClicked();
		void OnMnuSaveClicked();
		void OnMnuSaveAsClicked();
		void OnMnuExposureDownClicked();
		void OnMnuExposureUpClicked();
		void OnMnuBatchProcessingClicked();
		void OnMnuToneMappingClicked();
		void OnMnuAboutClicked();
		void OnMnuPreferencesClicked();
		void OnBrowseButtonClicked();
		void OnImageMouseMoved(GdkEventMotion* evt);
		void OnImageMouseDown(GdkEventButton* evt);
		void OnImageMouseUp(GdkEventButton* evt);
		
		void on_hide();
		
		void OnHdrGenerated();
		
		void OnFileDropped(const Glib::RefPtr<Gdk::DragContext>& context, int x, int y,
						   const Gtk::SelectionData& selection_data, guint info, guint time);
		//@}
		
		/// Used to capture close event to do additional stuff.
		bool on_delete_event(GdkEventAny* event);
		
	public:
		DisplayableImageForm(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& gladeBuilder);
		virtual ~DisplayableImageForm();
		
		void InitForm(bool move=true);
		bool OpenImage(const std::string& fileName);
		void Close() {this->OnMnuCloseClicked();}
		
		void SetPhoto(std::shared_ptr <PhotomatixImaging::PhotoImage> photo, bool init=true);
		std::shared_ptr <PhotomatixImaging::PhotoImage> GetPhoto();
		
		static DisplayableImageForm* MainForm;
		static DisplayableImageForm* CreateInstance();
		
		/// Sets initial zoom factor based on the image and screen dimensions.
		void SetOriginalZoom();
		void ShowCropRectangle(Gdk::Rectangle cropRect);
		float GetZoomFactor() const {return this->zoomFactor;}
		
		std::shared_ptr <HdrController> GetHdrController() {return this->hdrController;}
		void SetHdrController(std::shared_ptr <HdrController> controller) {this->hdrController = controller;}
		
		
		sigc::signal <void> signal_tonemapping_requested;
		sigc::signal <void, std::string> signal_file_open_requested;
		sigc::signal <void, GdkEventMotion*> signal_mouse_moved;
		sigc::signal <void, GdkEventButton*> signal_mouse_down;
		sigc::signal <void, GdkEventButton*> signal_mouse_up;
		
		
		/// Number of all currently opened images
		static int OpenedImagesCount;
	};
}

#endif
