#include "finishingtouch.h"
#include "../Photomatix32/Photomatix32.h"
#include "../Photomatix32/Utils.h"

#include <iomanip>


using namespace std;


namespace Photomatix
{
	FinishingTouch::FinishingTouch(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& gladeBuilder):
		Gtk::Window(cobject),
		builder(gladeBuilder),
		ratioVals({ 0.0f, 1.0f, 2.0f / 3.0f, 3.0f / 4.0f, 9.0f / 16.0f, 0.0f }),
		isCropped(false),
		isContrastCustom(false),
		isSharpeningCustom(false)
	{
		this->set_resizable(false);
		
		
		builder->get_widget("adjustment-tabs", this->adjustmentTabs);
		
		this->adjustmentTabs->signal_switch_page().connect(sigc::mem_fun(*this, &FinishingTouch::OnActiveTabChanged));
		
		
		builder->get_widget("btnDone", this->btnDone);
		builder->get_widget("btnReset", this->btnReset);
		builder->get_widget("btnApplyCrop", this->btnApplyCrop);
		
		btnDone->signal_clicked().connect(sigc::mem_fun(*this, &FinishingTouch::OnBtnDoneClicked));
		btnReset->signal_clicked().connect(sigc::mem_fun(*this, &FinishingTouch::OnBtnResetClicked));
		btnApplyCrop->signal_clicked().connect(sigc::mem_fun(*this, &FinishingTouch::OnBtnApplyCropClicked));
		
		
		
		builder->get_widget("cbxContrastPreset", this->cbxContrastPreset);
		builder->get_widget("cbxSharpeningPreset", this->cbxSharpeningPreset);
		builder->get_widget("cbxCropAspect", this->cbxCropAspect);
		
		this->cbxContrastPreset->set_row_separator_func(sigc::mem_fun(*this, &FinishingTouch::IsRowItemSeparator));
		this->cbxSharpeningPreset->set_row_separator_func(sigc::mem_fun(*this, &FinishingTouch::IsRowItemSeparator));
		
		this->contrastPresetSignal = this->cbxContrastPreset->signal_changed().connect(sigc::mem_fun(*this, &FinishingTouch::OnContrastPresetChanged));
		this->sharpeningPresetSignal = this->cbxSharpeningPreset->signal_changed().connect(sigc::mem_fun(*this, &FinishingTouch::OnSharpeningPresetChanged));
		this->cbxCropAspect->signal_changed().connect(sigc::mem_fun(*this, &FinishingTouch::OnCropAspectChanged));
		
		
		builder->get_widget("lblContrastShowOptions", this->lblContrastShowOptions);
		builder->get_widget("lblSharpeningShowOptions", this->lblSharpeningShowOptions);
		builder->get_widget("contrastExpander", this->contrastExpander);
		builder->get_widget("sharpeningExpander", this->sharpeningExpander);
		
		contrastExpander->property_expanded().signal_changed().connect(sigc::mem_fun(*this, &FinishingTouch::OnContrastExpanderChanged));
		sharpeningExpander->property_expanded().signal_changed().connect(sigc::mem_fun(*this, &FinishingTouch::OnSharpeningExpanderChanged));
		
		
		// Set up sliders
		builder->get_widget("sldHighlights", this->sldHighlights);
		builder->get_widget("sldLights", this->sldLights);
		builder->get_widget("sldDarks", this->sldDarks);
		builder->get_widget("sldShadows", this->sldShadows);
		builder->get_widget("sldAmount", this->sldAmount);
		builder->get_widget("sldRadius", this->sldRadius);
		builder->get_widget("sldThreshold", this->sldThreshold);
		
		builder->get_widget("lblHighlightsVal", this->lblHighlightsVal);
		builder->get_widget("lblLightsVal", this->lblLightsVal);
		builder->get_widget("lblDarksVal", this->lblDarksVal);
		builder->get_widget("lblShadowsVal", this->lblShadowsVal);
		builder->get_widget("lblAmountVal", this->lblAmountVal);
		builder->get_widget("lblRadiusVal", this->lblRadiusVal);
		builder->get_widget("lblThresholdVal", this->lblThresholdVal);
		
		
		
		this->sldHighlights->set_range(0, 100);
		this->sldLights->set_range(0, 100);
		this->sldDarks->set_range(-100, 0);
		this->sldShadows->set_range(-100, 0);
		this->sldAmount->set_range(0, 150);
		this->sldRadius->set_range(0.5, 3.0);
		this->sldThreshold->set_range(0, 20);
		
		this->sldHighlights->set_value(0);
		this->sldLights->set_value(0);
		this->sldDarks->set_value(0);
		this->sldShadows->set_value(0);
		this->sldAmount->set_value(0);
		this->sldRadius->set_value(1.0);
		this->sldThreshold->set_value(4);
		
		this->sldHighlights->set_round_digits(0);
		this->sldLights->set_round_digits(0);
		this->sldDarks->set_round_digits(0);
		this->sldShadows->set_round_digits(0);
		this->sldAmount->set_round_digits(0);
		this->sldRadius->set_round_digits(1);
		this->sldThreshold->set_round_digits(0);
		
		this->sldRadius->set_sensitive(false);
		this->sldThreshold->set_sensitive(false);
		this->lblRadiusVal->set_sensitive(false);
		this->lblThresholdVal->set_sensitive(false);
		
		this->sldHighlights->signal_button_release_event().connect(sigc::mem_fun(*this, &FinishingTouch::OnMouseUp), false);
		this->sldLights->signal_button_release_event().connect(sigc::mem_fun(*this, &FinishingTouch::OnMouseUp), false);
		this->sldDarks->signal_button_release_event().connect(sigc::mem_fun(*this, &FinishingTouch::OnMouseUp), false);
		this->sldShadows->signal_button_release_event().connect(sigc::mem_fun(*this, &FinishingTouch::OnMouseUp), false);
		this->sldAmount->signal_button_release_event().connect(sigc::mem_fun(*this, &FinishingTouch::OnMouseUp), false);
		this->sldRadius->signal_button_release_event().connect(sigc::mem_fun(*this, &FinishingTouch::OnMouseUp), false);
		this->sldThreshold->signal_button_release_event().connect(sigc::mem_fun(*this, &FinishingTouch::OnMouseUp), false);
		
		this->sldHighlightsSignal = this->sldHighlights->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &FinishingTouch::OnSliderValueChanging), this->sldHighlights));
		this->sldLightsSignal = this->sldLights->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &FinishingTouch::OnSliderValueChanging), this->sldLights));
		this->sldDarksSignal = this->sldDarks->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &FinishingTouch::OnSliderValueChanging), this->sldDarks));
		this->sldShadowsSignal = this->sldShadows->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &FinishingTouch::OnSliderValueChanging), this->sldShadows));
		this->sldAmountSignal = this->sldAmount->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &FinishingTouch::OnSliderValueChanging), this->sldAmount));
		this->sldRadiusSignal = this->sldRadius->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &FinishingTouch::OnSliderValueChanging), this->sldRadius));
		this->sldThresholdSignal = this->sldThreshold->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &FinishingTouch::OnSliderValueChanging), this->sldThreshold));
		
		this->sldHighlights->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &FinishingTouch::UpdateSliderValue), this->sldHighlights));
		this->sldLights->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &FinishingTouch::UpdateSliderValue), this->sldLights));
		this->sldDarks->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &FinishingTouch::UpdateSliderValue), this->sldDarks));
		this->sldShadows->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &FinishingTouch::UpdateSliderValue), this->sldShadows));
		this->sldAmount->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &FinishingTouch::UpdateSliderValue), this->sldAmount));
		this->sldRadius->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &FinishingTouch::UpdateSliderValue), this->sldRadius));
		this->sldThreshold->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &FinishingTouch::UpdateSliderValue), this->sldThreshold));
		
		this->OnContrastExpanderChanged();
		this->OnSharpeningExpanderChanged();
	}

	FinishingTouch::~FinishingTouch()
	{

	}
	
	bool FinishingTouch::IsRowItemSeparator(const Glib::RefPtr< Gtk::TreeModel >& model, const Gtk::TreeModel::iterator& iter)
	{
		Gtk::TreeModel::Row row = *iter;
		string value;
		row->get_value(0, value);
		if(value == "-")
			return true;
		
		return false;
	}
	
	void FinishingTouch::OnContrastExpanderChanged()
	{
		this->lblContrastShowOptions->set_text(this->contrastExpander->get_expanded() ? "Hide Options" : "Show Options");
	}

	void FinishingTouch::OnSharpeningExpanderChanged()
	{
		this->lblSharpeningShowOptions->set_text(this->sharpeningExpander->get_expanded() ? "Hide Options" : "Show Options");
	}

	void FinishingTouch::OnBtnDoneClicked()
	{
		this->close();
	}

	void FinishingTouch::OnBtnResetClicked()
	{
		this->ResetContrast();
		this->ResetSharpening();
		
		this->cbxContrastPreset->set_active(0);
		this->cbxSharpeningPreset->set_active(0);
		
		this->btnReset->set_sensitive(false);
		this->signal_settings_reset.emit();
	}
	
	void FinishingTouch::SetIsCropped(bool cropped)
	{
		this->isCropped = cropped;
		this->btnReset->set_sensitive(this->HasAdjustments() || this->isCropped);
	}

	
	void FinishingTouch::OnBtnApplyCropClicked()
	{
		this->signal_crop_applied.emit();
	}
	
	void FinishingTouch::ResetContrast()
	{
		sldShadows->set_value(0); sldDarks->set_value(0);
		sldLights->set_value(0); sldHighlights->set_value(0);
	}

	void FinishingTouch::ResetSharpening()
	{
		this->sldAmount->set_value(0);
		this->sldRadius->set_value(1.0);
		this->sldThreshold->set_value(4);
	}
	
	void FinishingTouch::OnContrastPresetChanged()
	{
		this->sldHighlightsSignal.block();
		this->sldLightsSignal.block();
		this->sldShadowsSignal.block();
		this->sldDarksSignal.block();
		
		if(this->isContrastCustom) {
			this->cbxContrastPreset->remove_text(0);
			this->isContrastCustom = false;
		}
		
		
		int index = this->cbxContrastPreset->get_active_row_number();
		if(index == 0) {
			this->ResetContrast();
		}
		else if (index == 2) {
			sldShadows->set_value(-25); sldDarks->set_value(0);
			sldLights->set_value(0); sldHighlights->set_value(15);
		}
		else if (index == 3) {
			sldShadows->set_value(-50); sldDarks->set_value(0);
			sldLights->set_value(0); sldHighlights->set_value(25);
		}
		else if(index == 4) {
			sldShadows->set_value(-75); sldDarks->set_value(-25);
			sldLights->set_value(25); sldHighlights->set_value(50);
		}
		else if(index == 5) {
			sldShadows->set_value(-100); sldDarks->set_value(-50);
			sldLights->set_value(50); sldHighlights->set_value(75);
		}
		
		this->UpdateContrastCurve();
		
		this->btnReset->set_sensitive(this->HasAdjustments() || this->isCropped);
		this->signal_settings_changed.emit();
		
		
		this->sldHighlightsSignal.unblock();
		this->sldLightsSignal.unblock();
		this->sldShadowsSignal.unblock();
		this->sldDarksSignal.unblock();
	}

	void FinishingTouch::OnSharpeningPresetChanged()
	{
		this->sldAmountSignal.block();
		this->sldRadiusSignal.block();
		this->sldThresholdSignal.block();
		
		if(this->isSharpeningCustom) {
			this->cbxSharpeningPreset->remove_text(0);
			this->isSharpeningCustom = false;
		}
		
		
		int index = this->cbxSharpeningPreset->get_active_row_number();
		if (index == 0) {
			this->ResetSharpening();
		}
		else if (index == 2) {
			this->sldAmount->set_value(50);
			this->sldRadius->set_value(1.0);
			this->sldThreshold->set_value(5);
		}
		else if (index == 3) {
			this->sldAmount->set_value(100);
			this->sldRadius->set_value(2.0);
			this->sldThreshold->set_value(5);
		}
		else if (index == 4) {
			this->sldAmount->set_value(150);
			this->sldRadius->set_value(3.0);
			this->sldThreshold->set_value(2);
		}
		
		bool isEnabled = this->sldAmount->get_value() > 0;
		this->sldRadius->set_sensitive(isEnabled);
		this->sldThreshold->set_sensitive(isEnabled);
		this->lblRadiusVal->set_sensitive(isEnabled);
		this->lblThresholdVal->set_sensitive(isEnabled);
		
			
		this->btnReset->set_sensitive(this->HasAdjustments() || this->isCropped);
		this->signal_settings_changed.emit();
		
		
		this->sldAmountSignal.unblock();
		this->sldRadiusSignal.unblock();
		this->sldThresholdSignal.unblock();
	}
	
	bool FinishingTouch::on_delete_event(GdkEventAny* event)
	{
		this->signal_finishingtouch_closed.emit();
		return false;
	}

	void FinishingTouch::OnSliderValueChanging(Gtk::Scale* sender)
	{
		if(sender == this->sldHighlights || sender == this->sldLights || sender == this->sldDarks || sender == this->sldShadows) {
			this->UpdateContrastCurve();
			
			
			this->contrastPresetSignal.block();
			
			if(this->isContrastCustom) {
				this->cbxContrastPreset->remove_text(0);
				this->isContrastCustom = false;
			}
			
			// Select preset (if current settings match any of them)
			if(sldShadows->get_value() == 0 && sldDarks->get_value() == 0 && sldLights->get_value() == 0 && sldHighlights->get_value() == 0) cbxContrastPreset->set_active(0);
			else if(sldShadows->get_value() == -25 && sldDarks->get_value() == 0 && sldLights->get_value() == 0 && sldHighlights->get_value() == 15) cbxContrastPreset->set_active(2);
			else if(sldShadows->get_value() == -50 && sldDarks->get_value() == 0 && sldLights->get_value() == 0 && sldHighlights->get_value() == 25) cbxContrastPreset->set_active(3);
			else if(sldShadows->get_value() == -75 && sldDarks->get_value() == -25 && sldLights->get_value() == 25 && sldHighlights->get_value() == 50) cbxContrastPreset->set_active(4);
			else if(sldShadows->get_value() == -100 && sldDarks->get_value() == -50 && sldLights->get_value() == 50 && sldHighlights->get_value() == 75) cbxContrastPreset->set_active(5);
			else {
				cbxContrastPreset->insert(0, "Custom");
				cbxContrastPreset->set_active(0);
				this->isContrastCustom = true;
			}
			
			this->contrastPresetSignal.unblock();
			
		} else if(sender == this->sldAmount || sender == this->sldRadius || sender == this->sldThreshold) {
			if(sender == this->sldAmount) {
				bool isEnabled = this->sldAmount->get_value() > 0;
				this->sldRadius->set_sensitive(isEnabled);
				this->sldThreshold->set_sensitive(isEnabled);
				this->lblRadiusVal->set_sensitive(isEnabled);
				this->lblThresholdVal->set_sensitive(isEnabled);
			}
			
			
			this->sharpeningPresetSignal.block();
			
			if(this->isSharpeningCustom) {
				this->cbxSharpeningPreset->remove_text(0);
				this->isSharpeningCustom = false;
			}
			
			// Select preset (if current settings match any of them)
			if(sldAmount->get_value() == 0 && sldRadius->get_value() == 1.0f && sldThreshold->get_value() == 4) cbxSharpeningPreset->set_active(0);
			else if(sldAmount->get_value() == 50 && sldRadius->get_value() == 1.0f && sldThreshold->get_value() == 5) cbxSharpeningPreset->set_active(2);
			else if(sldAmount->get_value() == 100 && sldRadius->get_value() == 2.0f && sldThreshold->get_value() == 5) cbxSharpeningPreset->set_active(3);
			else if(sldAmount->get_value() == 150 && sldRadius->get_value() == 3.0f && sldThreshold->get_value() == 2) cbxSharpeningPreset->set_active(4);
			else {
				cbxSharpeningPreset->insert(0, "Custom");
				cbxSharpeningPreset->set_active(0);
				this->isSharpeningCustom = true;
			}
			
			this->sharpeningPresetSignal.unblock();
		}
		
		this->btnReset->set_sensitive(this->HasAdjustments() || this->isCropped);
	}
	
	void FinishingTouch::UpdateSliderValue(Gtk::Scale* sender)
	{
		std::ostringstream str;
		str << std::fixed << std::setprecision(sender->get_digits()) << sender->get_value();
		
		if(sender == this->sldHighlights) this->lblHighlightsVal->set_text(str.str());
		else if(sender == this->sldLights) this->lblLightsVal->set_text(str.str());
		else if(sender == this->sldDarks) this->lblDarksVal->set_text(str.str());
		else if(sender == this->sldShadows) this->lblShadowsVal->set_text(str.str());
		else if(sender == this->sldAmount) this->lblAmountVal->set_text(str.str());
		else if(sender == this->sldRadius) this->lblRadiusVal->set_text(str.str());
		else if(sender == this->sldThreshold) this->lblThresholdVal->set_text(str.str());
	}
	
	void FinishingTouch::UpdateContrastCurve()
	{
		const int LutSize = 65536;
		if(this->contrastCurveLUT.empty()) this->contrastCurveLUT.resize(LutSize);
		
		std::vector<int> tmpLUT(LutSize);
		float contrastParam[] = { sldHighlights->get_value() / 100.0f, sldLights->get_value() / 100.0f,
			-sldShadows->get_value() / 100.0f, -sldDarks->get_value() / 100.0f };
		
		get_contrast_lut(contrastParam, tmpLUT.data());
		for (int i = 0; i < LutSize; ++i)
			this->contrastCurveLUT[i] = (unsigned short)tmpLUT[i];
	}
	
	void FinishingTouch::OnSliderValueChanged()
	{
		this->signal_settings_changed.emit();
	}

	bool FinishingTouch::OnMouseUp(GdkEventButton* evt)
	{
		this->OnSliderValueChanged();
		
		return false;
	}
	
	std::vector <unsigned short> FinishingTouch::GetContrastCurveLUT()
	{
		if(this->contrastCurveLUT.empty()) this->UpdateContrastCurve();
		
		return this->contrastCurveLUT;
	}
	
	float* FinishingTouch::GetSharpenParams()
	{
		this->sharpenParams[0] = sldAmount->get_value() >= 1.0f ? 1.0f + (sldAmount->get_value() / 100.0f) : 0.0f;
		this->sharpenParams[1] = sldRadius->get_value();
		this->sharpenParams[2] = sldThreshold->get_value();
		
		return this->sharpenParams;
	}
	
	bool FinishingTouch::HasAdjustments() const
	{
		return this->sldHighlights->get_value() >= 0.01f || this->sldLights->get_value() >= 0.01f || this->sldDarks->get_value() >= 0.01f || this->sldShadows->get_value() >= 0.01f ||
			this->sldAmount->get_value() > 0;
	}
	
	void FinishingTouch::OnActiveTabChanged(Gtk::Widget* widget, guint index)
	{
		this->signal_process_changed.emit(index);
	}
	
	void FinishingTouch::OnCropAspectChanged()
	{
		int idxCrop = this->cbxCropAspect->get_active_row_number();
		float aspectRatio = 0.0f;
		if (idxCrop >= 0 && idxCrop < this->ratioVals.size())
			aspectRatio = this->ratioVals[idxCrop];
		
		this->signal_crop_ratio_changed.emit(aspectRatio);
	}
	
	void FinishingTouch::SetCropControlEnability(bool canCrop)
	{
		this->btnApplyCrop->set_sensitive(canCrop);
	}


}
