#ifndef FINISHINGTOUCH_H
#define FINISHINGTOUCH_H


#include <gtkmm-3.0/gtkmm.h>


namespace Photomatix
{
	/** Finishing Touch window.
	*/
	class FinishingTouch: public Gtk::Window
	{	
	protected:
		Gtk::Notebook* adjustmentTabs;
		
		Gtk::ComboBoxText* cbxContrastPreset;
		Gtk::ComboBoxText* cbxSharpeningPreset;
		Gtk::ComboBox* cbxCropAspect;
		
		Gtk::Scale* sldHighlights;
		Gtk::Scale* sldLights;
		Gtk::Scale* sldDarks;
		Gtk::Scale* sldShadows;
		
		Gtk::Scale* sldAmount;
		Gtk::Scale* sldRadius;
		Gtk::Scale* sldThreshold;
		
		Gtk::Label* lblHighlightsVal;
		Gtk::Label* lblLightsVal;
		Gtk::Label* lblDarksVal;
		Gtk::Label* lblShadowsVal;
		Gtk::Label* lblAmountVal;
		Gtk::Label* lblRadiusVal;
		Gtk::Label* lblThresholdVal;
		
		Gtk::Label* lblContrastShowOptions;
		Gtk::Label* lblSharpeningShowOptions;
		
		Gtk::Expander* contrastExpander;
		Gtk::Expander* sharpeningExpander;
		
		Gtk::Button* btnDone;
		Gtk::Button* btnReset;
		Gtk::Button* btnApplyCrop;
		
		/// Glade builder used to access widgets
		Glib::RefPtr<Gtk::Builder> builder;
		
		sigc::connection contrastPresetSignal;
		sigc::connection sharpeningPresetSignal;
		
		sigc::connection sldHighlightsSignal;
		sigc::connection sldLightsSignal;
		sigc::connection sldDarksSignal;
		sigc::connection sldShadowsSignal;
		sigc::connection sldAmountSignal;
		sigc::connection sldRadiusSignal;
		sigc::connection sldThresholdSignal;
	
		std::vector <unsigned short> contrastCurveLUT;
		float sharpenParams[3];
		
		bool isCropped;
		bool isContrastCustom;
		bool isSharpeningCustom;
		
		
		std::vector <float> ratioVals;
		
		
		void UpdateContrastCurve();
		void ResetContrast();
		void ResetSharpening();
		void UpdateSliderValue(Gtk::Scale* sender);
		
		bool IsRowItemSeparator(const Glib::RefPtr<Gtk::TreeModel>&, const Gtk::TreeModel::iterator&);
		/// Used to capture close event to do additional stuff.
		bool on_delete_event(GdkEventAny* event);
		
		void OnActiveTabChanged(Widget* widget , guint index);
		void OnBtnDoneClicked();
		void OnBtnResetClicked();
		void OnBtnApplyCropClicked(); 
		void OnContrastPresetChanged();
		void OnSharpeningPresetChanged();
		void OnCropAspectChanged();
		void OnSliderValueChanging(Gtk::Scale* sender);
		void OnSliderValueChanged();
		bool OnMouseUp(GdkEventButton* evt);
		void OnContrastExpanderChanged();
		void OnSharpeningExpanderChanged();
		
	public:
		FinishingTouch(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& gladeBuilder);
		virtual ~FinishingTouch();
		
		std::vector <unsigned short> GetContrastCurveLUT();
		float* GetSharpenParams();
		
		void SetCropControlEnability(bool canCrop);
		
		sigc::signal <void, int> signal_process_changed;
		sigc::signal <void> signal_settings_changed;
		sigc::signal <void> signal_settings_reset;
		sigc::signal <void> signal_finishingtouch_closed;
		sigc::signal <void> signal_crop_applied;
		sigc::signal <void, float> signal_crop_ratio_changed;
		
		bool HasAdjustments() const;
		void SetIsCropped(bool cropped);
		
	};
}

#endif // FINISHINGTOUCH_H
