#include "hdrcontroller.h"
#include "hdrgenerator.h"
#include "load_bracketed_dialog.h"
#include "GladeUtils.h"
#include "preprocessingoptionsform.h"
#include "photoimage.h"
#include "progressform.h"
#if !defined _WIN32 && !defined __MACH__
	#include "../PhotomatixMisc/appproperties.h"
#endif
#include "progressnotifier.h"
#include "linuxprogressnotifier.h"


using namespace PhotomatixImaging;
using namespace PhotomatixMisc;
using namespace std;


namespace Photomatix
{
	HdrController::HdrController()
	{
	}

	HdrController::~HdrController()
	{

	}
	
	void HdrController::Init()
	{
		// First show the Load Bracketed Photos dialog
		this->loadBracketed = 0;
		PhotomatixUtils::get_widget_derived("load-bracketed-dialog", this->loadBracketed);
		if(this->loadBracketed)
		{
			this->loadBracketed->signal_ok.connect(sigc::mem_fun(*this, &HdrController::OnLoadBracketedOk));
			this->loadBracketed->run();
		}
	}
	
	void HdrController::Init(const std::vector <std::string>& files)
	{
		this->loadBracketed = 0;
		PhotomatixUtils::get_widget_derived("load-bracketed-dialog", this->loadBracketed);
		if(this->loadBracketed)
		{
			this->loadBracketed->signal_ok.connect(sigc::mem_fun(*this, &HdrController::OnLoadBracketedOk));
			for(auto& file : files)
			{
				if(PhotomatixImaging::PhotoImage::GetProbableFileFormat(file) != PhotomatixImaging::EImageFormatType::Unknown)
					this->loadBracketed->add_path_to_list(file);
			}
			this->loadBracketed->run();
		}
	}
	
	void HdrController::OnGeneratingHDRCanceled()
	{
		this->hdrGenerator->Stop();
	}
	
	void HdrController::GenerateHDR()
	{
		// Progress metering
		PhotomatixUtils::get_widget_derived("progress-dialog", this->progressForm);
#if !defined __MACH__ && !defined _WIN32
		std::string title = this->hdrGenerator->GetOutputFileName(AppProperties::GetConcatenateSourceFilenames());
#else
		std::string title = this->hdrGenerator->GetOutputFileName(true);
#endif
		
		this->progressForm->set_title(title.substr(title.find_last_of("//") + 1));
		this->progressForm->signal_stop_button_clicked.connect(sigc::mem_fun(*this, &HdrController::OnGeneratingHDRCanceled));
		this->hdrGenerator->progressNotifier = std::dynamic_pointer_cast<PhotomatixImaging::ProgressNotifier>(this->progressForm->GetProgressNotifier());
		if(this->progressForm)
			this->progressForm->show();	
		
		this->hdrGenerator->SetReduceChromaticAberrationsEnabled(this->preprocessingOptions->GetReduceChromaticAberrations());
		
		if(this->preprocessingOptions->GetAlignImages())
		{
			this->hdrGenerator->SetAlignmentEnabled(true);
			PhotomatixImaging::EAlignmentType type = PhotomatixImaging::EAlignmentType::AlignmentNone;
			if(this->preprocessingOptions->GetAlignByShifts())
				type = PhotomatixImaging::EAlignmentType::AlignmentTranslation;
			else if(this->preprocessingOptions->GetPerspectiveCorrection())
			{
				type = PhotomatixImaging::EAlignmentType::AlignmentRotationPerspective;
				this->hdrGenerator->SetMaxShiftPerspective(this->preprocessingOptions->GetMaximumShift());
			}
			else
				type = PhotomatixImaging::EAlignmentType::AlignmentRotation;
			
			this->hdrGenerator->SetAlignmentType(type);
			this->hdrGenerator->SetCropImagesEnabled(this->preprocessingOptions->GetCropAlignedImages());
		}
		else
			this->hdrGenerator->SetAlignmentEnabled(false);
		
		if(this->preprocessingOptions->GetDeghost())
		{
			this->hdrGenerator->SetGhostRemovalEnabled(true);
			this->hdrGenerator->SetGhostRemovalDetectionStrength(this->preprocessingOptions->GetGhostingDetectionStrength());
		}
		else
			this->hdrGenerator->SetGhostRemovalEnabled(false);
		
		if(this->preprocessingOptions->GetReduceNoise())
		{
			this->hdrGenerator->SetNoiseReductionType(this->preprocessingOptions->GetNoiseReductionType());
			this->hdrGenerator->SetNoiseReductionStrength(this->preprocessingOptions->GetNoiseReductionStrength());
		}
		else
			this->hdrGenerator->SetNoiseReductionType(PhotomatixImaging::ENoiseReductionType::NoiseReductionNone);
		
		this->hdrGenerator->SetColorTemperature(this->preprocessingOptions->GetColorTemperature());
		this->hdrGenerator->SetColorSpace(this->preprocessingOptions->GetColorSpace());
		
		// Start generating HDR
		this->hdrGenerator->Start();
	}
	
	void HdrController::OnGeneratingHDRDone()
	{
		// First get rid of the progress form as we don't need it anymore
		this->progressForm->hide();
		delete progressForm;
			
		if(!this->hdrGenerator->Cancelled())
		{
			DisplayableImageForm* form = this->imageWindow;
			form->SetPhoto(this->hdrGenerator->GetOutputPhoto());
			form->hide();
			form->signal_tonemapping_requested();
			
			this->signal_hdr_generated.emit();
		}
	}
	
	void HdrController::OnLoadBracketedOk()
	{
		// Start merging photos into HDR
		this->hdrGenerator = shared_ptr<HdrGenerator>(new HdrGenerator());
		this->hdrGenerator->sig_done.connect(sigc::mem_fun(*this, &HdrController::OnGeneratingHDRDone));
		std::vector <std::string> fileNames = this->loadBracketed->GetFileNames();
		if(fileNames.size() <= 1)
			return;
		this->hdrGenerator->LoadInputPhotos(fileNames);
		if(!this->hdrGenerator->CheckPhotosIntegrity())
			return;
		
		this->loadBracketed->hide();
		
		// Show preprocessing options form
		PhotomatixUtils::get_widget_derived("preprocessing-dialog", this->preprocessingOptions);
		this->preprocessingOptions->SetSourceImagesRaw(this->hdrGenerator->GetInputPhoto(0)->GetImageFormat() == EImageFormatType::Raw);
		this->preprocessingOptions->signal_ok.connect(sigc::mem_fun(*this, &HdrController::GenerateHDR));
		if(this->preprocessingOptions)
			this->preprocessingOptions->run();
	}
}