#include "presetswindow.h"
#include "presetthumbnail.h"
#include "../PhotomatixImaging/tonemapper.h"
#include "../PhotomatixImaging/preset.h"
#include "../PhotomatixImaging/exposurefusionprocessor.h"
#include "../PhotomatixImaging/photoimage.h"
#include "../Photomatix32/Photomatix32.h"
#include "../Photomatix32/Photomatix40Lib1.h"
#include "../PhotomatixMisc/appproperties.h"

using namespace std;
using namespace PhotomatixImaging;
using namespace PhotomatixMisc;


namespace Photomatix
{
	PresetThumbnailsGenerator::PresetThumbnailsGenerator(PresetsWindow* presetsWindow, Queue <Thumbnail>& queue):
		caller(presetsWindow),
		outputQueue(queue),
		stopped(false),
		shallStop(false)
	{
	}
	
	void PresetThumbnailsGenerator::Stop()
	{
		std::lock_guard<std::mutex> lock(mutexStopped_);
		shallStop = true;
	}
	
	void PresetThumbnailsGenerator::Process()
	{
		ToneMapper mapper;
		ExposureFusionProcessor fusion;
		
		{
			std::lock_guard<std::mutex> lock(mutexStopped_);
			stopped = false;
		}
		
		std::vector <std::shared_ptr <PhotoImage>> photos;
		if(photo != NULL)
			photos.push_back(photo);
		
		
		for(Preset& preset : presetsList)
		{
			//PresetThumbnail* thumb = new PresetThumbnail();
			shared_ptr <PhotoImage> thumbnail;
			//thumb->SetSettings(preset);
			
			// Render the thumbnail using Tone-Mapper or Fusion
			if(preset.GetMethod() <= 4)
			{				
				/*ToneMapper* mapper = new ToneMapper();
				mapper->SetIsPreview(true);
				mapper->SetToneMappingMethod(preset.GetMethod() == 1 ? PhotomatixImaging::EToneMappingMethod::TM_DETAILS_ENHANCER : (preset.GetMethod() == 2 ? PhotomatixImaging::EToneMappingMethod::TM_TONE_COMPRESSOR : PhotomatixImaging::EToneMappingMethod::TM_CONTRAST_OPTIMIZER));
				mapper->SetBitsPerSampleOut(8);
				mapper->SetLoadImages(false);
				mapper->SetInputPhotos(photos);
				mapper->SetProcessingSettings(preset.GetSettings()->parameters);
				mapper->StartSync();*/
				
				
				mapper.SetIsPreview(true);
				mapper.SetToneMappingMethod(preset.GetMethod() == 1 ? PhotomatixImaging::EToneMappingMethod::TM_DETAILS_ENHANCER : (preset.GetMethod() == 2 ? PhotomatixImaging::EToneMappingMethod::TM_TONE_COMPRESSOR : PhotomatixImaging::EToneMappingMethod::TM_CONTRAST_OPTIMIZER));
				mapper.SetBitsPerSampleOut(8);
				mapper.SetInputPhotos(photos);
				mapper.SetLoadImages(false);
				mapper.SetProcessingSettings(preset.GetSettings()->parameters);
				mapper.StartSync();
				
				thumbnail = mapper.GetOutputPhoto();
			}
			else if(preset.GetMethod() == 10 && caller->imageBuffer->size())
			{
				fusion.SetIsPreview(true);
				fusion.SetExposureFusionMethod(EFusionMethod::FusionAdjust);
				fusion.SetBitsPerSampleOut(8);
				fusion.SetInputPhotos(caller->sourcePhotos);
				fusion.SetImagesBuffer(caller->imageBuffer);
				fusion.SetLoadImages(false);
				fusion.SetProcessingSettings(preset.GetSettings()->parameters);
				fusion.SetGhostMask(caller->ghostMask);
				if(caller->ghostMask)
					fusion.SetGhostRemovalEnabled(true);
				fusion.StartSync();
				
				thumbnail = fusion.GetOutputPhoto();
			}
			
			// Thread was stopped by caller
			if(shallStop)
			{
				break;
			}
			
			if(thumbnail)
			{
				// Add thumbnail to the queue of rendered thumbnails
				this->outputQueue.push(std::make_pair(thumbnail, preset));
				
				// Notify GUI thread that new thumbnail has been created
				caller->NotifyThumbnailCreated();
			}
		}
		
		{
			std::lock_guard<std::mutex> lock(mutexStopped_);
			stopped = true;
			shallStop = false;
		}
		
		// Notify GUI thread that we're done rendering thumbnails
		caller->NotifyFinished();
	}
	
	bool PresetThumbnailsGenerator::HasStopped() const
	{
		std::lock_guard<std::mutex> lock(mutexStopped_);
		return stopped;
	}
	
	
	/*void PresetThumbnailsGeneratorConsumer::operator()()
	{
		std::cout << "!!!START!!!\n";
		
		int currentPreset = 0;
		
		while(currentPreset < presetsNum) {
			PresetThumbnail* thumbnailImg = outputQueue.pop();
			
			std::cout << "functor\n";
			
			++currentPreset;
		}
		
		std::cout << "!!!DONE!!!\n";
	}*/
	
	
	PresetsWindow::PresetsWindow():
		thumbnailGenerator(this, thumbnailGeneratorQueue),
		thumbnailGeneratorThread(nullptr)
	{
		this->set_size_request(120, 400);
		this->set_redraw_on_allocate(false);
		this->set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);
		this->set_border_width(5);
		
		
		dispatcher.connect(sigc::mem_fun(*this, &PresetsWindow::OnThumbnailCreated));
		dispatcherFinished.connect(sigc::mem_fun(*this, &PresetsWindow::OnThumbnailsCreated));
		
		
		
		Gtk::Box* boxes[] = {&this->boxPresetsBuiltIn, &this->boxPresetsCustom};
		
		for(Gtk::Box* b : boxes)
		{
			b->set_orientation(Gtk::ORIENTATION_VERTICAL);
			b->set_homogeneous(true);
		
			b->set_size_request(120, 100);
			b->set_homogeneous(false);
			b->set_spacing(5);
		}
		
		this->lblNoCustomPresets.set_text("You do not have saved presets yet.\n\nTo create a preset from your current settings, click on the Preset combo-box (located at bottom of the Settings Adjustment window), select 'Save Preset…' and save in the default location.");
		this->lblNoCustomPresets.set_line_wrap(true);
		this->lblNoCustomPresets.set_margin_top(100);
		//this->lblNoCustomPresets.set_line_wrap_mode();
		//this->set_size_request(120, 10);
		
		
		this->isShowingBuiltIn = PhotomatixMisc::AppProperties::GetBuiltInPresetsSelected();
		this->refreshBuiltInPresets = !PhotomatixMisc::AppProperties::GetBuiltInPresetsSelected();
		this->refreshCustomPresets = PhotomatixMisc::AppProperties::GetBuiltInPresetsSelected();
		if(PhotomatixMisc::AppProperties::GetBuiltInPresetsSelected()) {
			this->add(this->boxPresetsBuiltIn);
		}
		else {
			this->add(this->boxPresetsCustom);
		}
	}

	PresetsWindow::~PresetsWindow()
	{
		this->StopThumbnailsRendering();
	}
	
	void PresetsWindow::StopThumbnailsRendering()
	{
		this->thumbnailGenerator.Stop();
		if (this->thumbnailGeneratorThread && this->thumbnailGeneratorThread->joinable()) {
			this->thumbnailGeneratorThread->join();
		}
		delete this->thumbnailGeneratorThread;
		this->thumbnailGeneratorThread = nullptr;
	}
	
	void PresetsWindow::RefreshCustomPresets()
	{
		this->refreshCustomPresets = true;
	}
	
	void PresetsWindow::ShowPresets(bool builtIn)
	{
		this->isShowingBuiltIn = builtIn;
		if((builtIn && this->refreshBuiltInPresets) || (!builtIn && this->refreshCustomPresets))
		{
			this->RenderPresets(builtIn);
			
			if(builtIn && this->refreshBuiltInPresets) this->refreshBuiltInPresets = false;
			else this->refreshCustomPresets = false;
		}
		
		this->remove_with_viewport();
		
		this->add(builtIn ? this->boxPresetsBuiltIn : this->boxPresetsCustom);
		this->show_all_children();
	}
	
	void PresetsWindow::NotifyThumbnailCreated()
	{
		dispatcher.emit();
	}
	
	void PresetsWindow::NotifyFinished()
	{
		dispatcherFinished.emit();
	}
	
	void PresetsWindow::RenderPresets(bool builtIn)
	{
		signal_thumbnails_rendering_changed.emit(false);
		
		// Get presets list
		this->GetPresetsList(builtIn);
		
		
		std::vector <Preset> presets = builtIn ? this->presetsList : this->presetsListCustom;
		// No presets, so skip the rest. This can happen if user has no custom presets installed, yet he selected My Presets tab
		if(presets.empty())
		{
			if(!builtIn) {
				this->boxPresetsCustom.add(this->lblNoCustomPresets);
			}
			
			signal_thumbnails_rendering_changed.emit(true);
			return;
		}
		
		Gtk::Box& boxControl = builtIn ? boxPresetsBuiltIn : boxPresetsCustom;
		// Remove all existing thumbnails from the box control
		Glib::ListHandle<Widget*> childList = boxControl.get_children();
		Glib::ListHandle<Widget*>::iterator it = childList.begin();

		while (it != childList.end()) {
			boxControl.remove(*(*it));
			it++;
		}
		
		this->thumbnailGenerator.SetPresetsList(presets);
		this->thumbnailGenerator.SetSourcePhoto(inputPhoto);
		
		this->thumbnailGeneratorThread = new thread(
			[this]
			{
				thumbnailGenerator.Process();
			});
		//thumbnailGenerator.Process();
		
		//PresetThumbnailsGeneratorConsumer consumerFunctor(this->thumbnailGeneratorQueue, presets.size());
		//consumerThread = new std::thread(consumerFunctor);
		
		
		// Render the thumbnails
		/*int index = 0;
		for(Preset& preset : presets)
		{
			if(preset.GetMethod() <= 4 || preset.GetMethod() == 10)
			{
				shared_ptr <PhotoImage> photo = inputPhoto;
				shared_ptr <PhotoImage> outputPhoto;
				if(preset.GetMethod() <= 4)
				{
					ToneMapper mapper;
					mapper.SetIsPreview(true);
					mapper.SetToneMappingMethod(preset.GetMethod() == 1 ? PhotomatixImaging::EToneMappingMethod::TM_DETAILS_ENHANCER : (preset.GetMethod() == 2 ? PhotomatixImaging::EToneMappingMethod::TM_TONE_COMPRESSOR : PhotomatixImaging::EToneMappingMethod::TM_CONTRAST_OPTIMIZER));
					mapper.SetBitsPerSampleOut(8);
					std::vector <std::shared_ptr <PhotoImage>> photos;
					if(photo != NULL)
						photos.push_back(photo);
					mapper.SetInputPhotos(photos);
					mapper.SetLoadImages(false);
					mapper.SetProcessingSettings(preset.GetSettings()->parameters);
					//mapper.sig_done.connect(sigc::mem_fun(*this, &PreviewController::OnPreviewGenerationDone));
					mapper.StartSync();
					
					outputPhoto = mapper.GetOutputPhoto();
				}
				else if(preset.GetMethod() == 10 && imageBuffer->size())
				{
					ExposureFusionProcessor fusion;
					fusion.SetIsPreview(true);
					fusion.SetExposureFusionMethod(EFusionMethod::FusionAdjust);
					fusion.SetBitsPerSampleOut(8);
					fusion.SetInputPhotos(sourcePhotos);
					fusion.SetImagesBuffer(imageBuffer);
					fusion.SetLoadImages(false);
					fusion.SetProcessingSettings(preset.GetSettings()->parameters);
					fusion.SetGhostMask(ghostMask);
					if(ghostMask)
						fusion.SetGhostRemovalEnabled(true);
					fusion.StartSync();
					
					outputPhoto = fusion.GetOutputPhoto();
				}
				
				if(outputPhoto)
				{
					PresetThumbnail* thumb = new PresetThumbnail();				
					thumb->SetThumbnailImage(outputPhoto);
					thumb->SetSettings(preset);
					thumb->signal_thumbnail_clicked.connect(sigc::mem_fun(*this, &PresetsWindow::OnThumbnailClicked));
					
					boxControl.pack_start(*thumb, Gtk::PACK_SHRINK);
				}
			}
			// We don't support other processing methods (at least in 1.0)
			
			++index;
		}*/
	}
	
	void PresetsWindow::InitPresetsThumbnails(shared_ptr <PhotoImage> inputPhoto, vector <shared_ptr <PhotoImage>> sourcePhotos, shared_ptr <vector <unsigned char> > imageBuffer, shared_ptr <vector <unsigned char> > ghostMask)
	{	
		// TODO: run this code in the worker thread
		
		this->inputPhoto = inputPhoto;
		this->sourcePhotos = sourcePhotos;
		this->imageBuffer = imageBuffer;
		this->ghostMask = ghostMask;
		
		//this->RenderPresets(true);
		this->RenderPresets(this->isShowingBuiltIn);
	}
	
	void PresetsWindow::OnThumbnailsCreated()
	{
		if (this->thumbnailGeneratorThread && this->thumbnailGenerator.HasStopped())
		{
			// Work is done.
			if (this->thumbnailGeneratorThread->joinable())
				this->thumbnailGeneratorThread->join();
			delete this->thumbnailGeneratorThread;
			this->thumbnailGeneratorThread = nullptr;
			//update_start_stop_buttons();
			
			signal_thumbnails_rendering_changed.emit(true);
		}
	}

	
	void PresetsWindow::OnThumbnailCreated()
	{
		Thumbnail /*thumbnailImg*/thumbnail = this->thumbnailGeneratorQueue.pop();
		
		//if(thumb)
		//{
		/*	PresetThumbnail* thumb = new PresetThumbnail();				
			thumb->SetThumbnailImage(thumbnailImg);
			thumb->SetSettings(preset);*/
			PresetThumbnail* thumb = new PresetThumbnail();				
			thumb->SetThumbnailImage(thumbnail.first);
			thumb->SetSettings(thumbnail.second);
			thumb->signal_thumbnail_clicked.connect(sigc::mem_fun(*this, &PresetsWindow::OnThumbnailClicked));
			
			Gtk::Box& boxControl = this->isShowingBuiltIn ? boxPresetsBuiltIn : boxPresetsCustom;
			boxControl.pack_start(*thumb, Gtk::PACK_SHRINK);
			boxControl.show_all_children();
		//}
	}
	
	void PresetsWindow::OnThumbnailClicked(PresetThumbnail* thumb)
	{
		this->signal_settings_changed.emit(&thumb->GetPreset());
	}
	
	void PresetsWindow::GetPresetsList(bool builtIn)
	{
		if(builtIn) {
			this->presetsList.clear();
			
			// Load built-in presets
			this->presetsList.push_back(Preset("Detailed", "", true, 1));
			this->presetsList.push_back(Preset("Balanced", "", true, 3));
			this->presetsList.push_back(Preset("Realistic", "", true, 4));
			this->presetsList.push_back(Preset("Photographic", "", true, 2));
			this->presetsList.push_back(Preset("Natural", "", true, 10));
			this->presetsList.push_back(Preset("Painterly", "BuiltinPresets/Painterly.xmp"));
			this->presetsList.push_back(Preset("Painterly 2", "BuiltinPresets/Painterly2.xmp"));
			this->presetsList.push_back(Preset("Painterly 3", "BuiltinPresets/Painterly3.xmp"));
			this->presetsList.push_back(Preset("Painterly 4", "BuiltinPresets/Painterly4.xmp"));
			this->presetsList.push_back(Preset("Painterly 5", "BuiltinPresets/Painterly5.xmp"));
			this->presetsList.push_back(Preset("Vibrant", "BuiltinPresets/Vibrant.xmp"));
			this->presetsList.push_back(Preset("Surreal", "BuiltinPresets/Surreal.xmp"));
			this->presetsList.push_back(Preset("Surreal 2", "BuiltinPresets/Surreal2.xmp"));
			this->presetsList.push_back(Preset("Grunge", "BuiltinPresets/Grunge.xmp"));
			this->presetsList.push_back(Preset("Grunge 2", "BuiltinPresets/Grunge2.xmp"));
			this->presetsList.push_back(Preset("Creative", "BuiltinPresets/Creative.xmp"));
			this->presetsList.push_back(Preset("Creative 2", "BuiltinPresets/Creative2.xmp"));
			this->presetsList.push_back(Preset("Creative 3", "BuiltinPresets/Creative3.xmp"));
			this->presetsList.push_back(Preset("Enhanced", "BuiltinPresets/Enhanced.xmp"));
			this->presetsList.push_back(Preset("Enhanced 2", "BuiltinPresets/Enhanced2.xmp"));
			this->presetsList.push_back(Preset("Realistic 2", "BuiltinPresets/Realistic2.xmp"));
			this->presetsList.push_back(Preset("Soft", "BuiltinPresets/Soft.xmp"));
			this->presetsList.push_back(Preset("Soft 2", "BuiltinPresets/Soft2.xmp"));
			this->presetsList.push_back(Preset("Soft 3", "BuiltinPresets/Soft3.xmp"));
			this->presetsList.push_back(Preset("Soft 4", "BuiltinPresets/Soft4.xmp"));
			this->presetsList.push_back(Preset("Smooth", "BuiltinPresets/Smooth.xmp"));
			this->presetsList.push_back(Preset("Smooth 2", "BuiltinPresets/Smooth2.xmp"));
			this->presetsList.push_back(Preset("Interior 4", "BuiltinPresets/Interior4.xmp"));
			this->presetsList.push_back(Preset("Deep", "BuiltinPresets/CompressorDeep.xmp"));
			this->presetsList.push_back(Preset("Vibrant 2", "BuiltinPresets/Vibrant2.xmp"));
			this->presetsList.push_back(Preset("Monochrome", "BuiltinPresets/DefaultBW.xmp"));
			this->presetsList.push_back(Preset("Monochrome 2", "BuiltinPresets/BW2.xmp"));
			this->presetsList.push_back(Preset("Monochrome 3", "BuiltinPresets/BW3.xmp"));
			this->presetsList.push_back(Preset("B&W Realistic", "BuiltinPresets/RealisticBW.xmp"));
			this->presetsList.push_back(Preset("B&W Photographic", "BuiltinPresets/PhotographicBW.xmp"));
			this->presetsList.push_back(Preset("B&W Artistic", "BuiltinPresets/ArtisticBW.xmp"));
			this->presetsList.push_back(Preset("B&W Natural", "BuiltinPresets/FusionBW.xmp"));
			//this->presetsList.push_back(Preset("Adjusted", "BuiltinPresets/FusionAdjusted.xmp"));
		} else {
			this->presetsListCustom.clear();
			
			string path = PhotomatixMisc::AppProperties::GetUserPath() + "custom_presets/";
			
			// Make sure to create custom presets path if it doesn't exist yet
			struct stat sb;
			if (!stat(path.c_str(), &sb) == 0 || !S_ISDIR(sb.st_mode))
				mkdir(path.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
			
			// Load custom presets
			Glib::RefPtr<Gio::File> file = Gio::File::create_for_path(path);
			Glib::RefPtr<Gio::FileEnumerator> child_enumeration = file->enumerate_children(G_FILE_ATTRIBUTE_STANDARD_NAME);
			std::vector<Glib::ustring> file_names;
			Glib::RefPtr<Gio::FileInfo> file_info;
			
			while ((file_info = child_enumeration->next_file()) != NULL)
			{
				string displayName = file_info->get_name();
				if(displayName.find_last_of(".") != string::npos) {
					displayName = std::string(displayName.begin(), displayName.begin() + displayName.find_last_of("."));
					this->presetsListCustom.push_back(Preset(displayName, path + file_info->get_name(), false));
				}
			}
			std::sort(this->presetsListCustom.begin(), this->presetsListCustom.end(), [](Preset preset1, Preset preset2) {
				string lhs = preset1.GetDisplayName();
				string rhs = preset2.GetDisplayName();
				
				std::transform(lhs.begin(), lhs.end(), lhs.begin(), ::toupper);
				std::transform(rhs.begin(), rhs.end(), rhs.begin(), ::toupper);
				
				return lhs < rhs;
			});
		}
		
		bool hasFusion = this->imageBuffer->size();
		std::vector <PhotomatixImaging::Preset>& presets = builtIn ? this->presetsList : this->presetsListCustom;
		// Remove fusion presets from the list if fusion is not available (for single photos)
		if(!hasFusion) {
			presets.erase(std::remove_if(presets.begin(), presets.end(),
						     [](PhotomatixImaging::Preset p) { return p.GetMethod() >= 10; }), presets.end());
		}
	}

}
