#ifndef SINGLE_RAW_CONVERSION_DIALOG_H
#define SINGLE_RAW_CONVERSION_DIALOG_H

#include <gtkmm.h>
#include "preprocessingoptionsform.h"


namespace Photomatix
{
	// Dialog for converting single RAW photos.
	class SingleRawConversionDialog: public Gtk::Dialog
	{
	protected:
		Gtk::Button *btnCancel;
		Gtk::Button *btnOk;
		Gtk::CheckButton *chkReduceNoise;
		Gtk::ComboBox *cbxColorSpace;
		Gtk::ComboBox *cbxWhiteBalance;
		Gtk::Entry *txtCustomTemperature;
		Gtk::HScale *sldReduceNoiseStrength;
		Gtk::Label *label1;
		Gtk::Label *label2;
		Gtk::Label *label3;
		
		EWhiteBalance whiteBalanceSettings[8];
		
		
		void OnOkButtonClicked();
		void OnReduceNoiseToggled();
		void OnWhiteBalanceSelectedIndexChanged();
		bool OnCustomTemperatureLostFocus(GdkEventFocus* evt);
		
		void ReadStickySettings();
		void SaveStickySettings();
		
	public:
		SingleRawConversionDialog(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder);
		virtual ~SingleRawConversionDialog();
		
		
		bool GetReduceNoise() const { return this->chkReduceNoise->get_active(); }
		double GetReduceNoiseStrength() const { return this->sldReduceNoiseStrength->get_value(); }
		
		int GetNoiseReductionStrength() const {return this->sldReduceNoiseStrength->get_value();}
		
		int GetColorTemperature() const
		{
			if(this->whiteBalanceSettings[this->cbxWhiteBalance->get_active_row_number()] != EWhiteBalance::WBCustom)
				return this->whiteBalanceSettings[this->cbxWhiteBalance->get_active_row_number()];
			else
				return atoi(this->txtCustomTemperature->get_text().c_str());
		}
		
		unsigned int GetColorSpace() const {return this->cbxColorSpace->get_active_row_number();}
	};
}


#endif //LOAD_BRACKETED_DIALOG_H
