#include "detailsenhancersettings.h"

#include <iostream>
#include <iomanip>
#include <math.h>

using namespace Photomatix;
using namespace PhotomatixImaging;


namespace PhotomatixControls
{
	DetailsEnhancerSettings::DetailsEnhancerSettings(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder):
		ProcessingSettingsControl(cobject, builder)
	{
		builder->get_widget("btnNaturalPlus", this->btnNaturalPlus);
		builder->get_widget("btnNatural", this->btnNatural);
		builder->get_widget("btnMedium", this->btnMedium);
		builder->get_widget("btnSurreal", this->btnSurreal);
		builder->get_widget("btnSurrealPlus", this->btnSurrealPlus);
		
		builder->get_widget("tabs", tabs);
		
		
		// Set tabs background colour to transparent (otherwise it might be very bright on some systems)
		Gdk::RGBA bgColour;
		bgColour.set_alpha(0);
		this->tabs->override_background_color(bgColour);
		
		builder->get_widget("lighting-slider-box", this->boxSlider);
		builder->get_widget("lighting-buttons-box", this->boxButtons);
		builder->get_widget("chkLightingEffectsMode", this->chkLightingEffectsMode);
		builder->get_widget("chkLightingEffects", this->chkLightingEffects);
		
		// Change font
		Pango::FontDescription fontDesc;
		fontDesc.set_family("Liberation Sans");
		fontDesc.set_size(7 * PANGO_SCALE);
		
		chkLightingEffects->get_child()->override_font(fontDesc);
		chkLightingEffectsMode->get_child()->override_font(fontDesc);
		
		this->effectsModeSignal = this->chkLightingEffectsMode->signal_toggled().connect(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnLightingEffectsModeToggled));
		this->lightingAdjustmentsSignal = this->chkLightingEffects->signal_toggled().connect(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnLightingEffectsToggled));
		
		// Set up sliders
		builder->get_widget("sldStrength", this->sldStrength);
		builder->get_widget("sldSaturation", this->sldSaturation);
		builder->get_widget("sldLuminosity", this->sldLuminosity);
		builder->get_widget("sldDetailContrast", this->sldDetailContrast);
		builder->get_widget("sldLightingAdjustments", this->sldLightingAdjustments);
		builder->get_widget("sldSmoothHighlights", this->sldSmoothHighlights);
		builder->get_widget("sldWhitePoint", this->sldWhitePoint);
		builder->get_widget("sldBlackPoint", this->sldBlackPoint);
		builder->get_widget("sldGamma", this->sldGamma);
		builder->get_widget("sldTemperature", this->sldTemperature);
		builder->get_widget("sldMicroSmoothing", this->sldMicroSmoothing);
		builder->get_widget("sldSaturationHighlights", this->sldSaturationHighlights);
		builder->get_widget("sldSaturationShadows", this->sldSaturationShadows);
		builder->get_widget("sldShadowsSmoothness", this->sldShadowsSmoothness);
		builder->get_widget("sldShadowsClipping", this->sldShadowsClipping);
		builder->get_widget("chk360image", this->chk360image);
		
		builder->get_widget("lblStrength", this->lblStrength);		
		builder->get_widget("lblSaturation", this->lblSaturation);
		builder->get_widget("lblLuminosity", this->lblLuminosity);
		builder->get_widget("lblDetailContrast", this->lblDetailContrast);
		builder->get_widget("lblLightingAdjustments", this->lblLightingAdjustments);
		builder->get_widget("lblSmoothHighlights", this->lblSmoothHighlights);
		builder->get_widget("lblWhitePoint", this->lblWhitePoint);
		builder->get_widget("lblBlackPoint", this->lblBlackPoint);
		builder->get_widget("lblGamma", this->lblGamma);
		builder->get_widget("lblTemperature", this->lblTemperature);
		builder->get_widget("lblMicroSmoothing", this->lblMicroSmoothing);
		builder->get_widget("lblSaturationHighlights", this->lblSaturationHighlights);
		builder->get_widget("lblSaturationShadows", this->lblSaturationShadows);
		builder->get_widget("lblShadowsSmoothness", this->lblShadowsSmoothness);
		builder->get_widget("lblShadowsClipping", this->lblShadowsClipping);
		
		
		this->sldStrength->set_range(0, 100);
		this->sldStrength->set_digits(0);
		this->sldSaturation->set_range(-10.0, 10);
		this->sldSaturation->set_digits(1);
		this->sldLuminosity->set_range(-10.0, 10.0);
		this->sldDetailContrast->set_range(-10.0, 10.0);
		this->sldLightingAdjustments->set_range(-10.0, 10.0);
		this->sldSmoothHighlights->set_range(0, 100);
		this->sldSmoothHighlights->set_digits(0);
		this->sldGamma->set_range(-10.0, 10.0);
		this->sldGamma->set_digits(1);
		this->sldTemperature->set_range(-10.0, 10.0);
		this->sldMicroSmoothing->set_range(0.0, 30.0);
		this->sldSaturationHighlights->set_range(-10.0, 10.0);
		this->sldSaturationShadows->set_range(-10.0, 10.0);
		this->sldShadowsSmoothness->set_range(0, 100);
		this->sldShadowsSmoothness->set_digits(0);
		this->sldShadowsClipping->set_range(0, 100);
		this->sldShadowsClipping->set_digits(0);
		this->sldWhitePoint->set_range(0.0, 10.0);
		this->sldWhitePoint->set_digits(3);
		this->sldBlackPoint->set_range(0.0, 10.0);
		this->sldBlackPoint->set_digits(3);
		
		this->sldStrength->set_value(70);
		this->sldSaturation->set_value(0);
		this->sldWhitePoint->set_value(0.25f);
		this->sldGamma->set_value(0);
		this->sldMicroSmoothing->set_value(2.0f);
		
		
		this->sldStrength->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldSaturation->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldLuminosity->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldDetailContrast->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldLightingAdjustments->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldSmoothHighlights->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldGamma->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldTemperature->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldMicroSmoothing->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldSaturationHighlights->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldSaturationShadows->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldShadowsSmoothness->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldShadowsClipping->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldWhitePoint->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldBlackPoint->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		
		this->chk360image->signal_toggled().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnSliderValueChanged));
		
		
		this->btnNaturalPlus->signal_clicked().connect(sigc::bind<Gtk::ToggleButton*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnButtonClicked), this->btnNaturalPlus));
		this->btnNatural->signal_clicked().connect(sigc::bind<Gtk::ToggleButton*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnButtonClicked), this->btnNatural));
		this->btnMedium->signal_clicked().connect(sigc::bind<Gtk::ToggleButton*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnButtonClicked), this->btnMedium));
		this->btnSurreal->signal_clicked().connect(sigc::bind<Gtk::ToggleButton*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnButtonClicked), this->btnSurreal));
		this->btnSurrealPlus->signal_clicked().connect(sigc::bind<Gtk::ToggleButton*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnButtonClicked), this->btnSurrealPlus));
		
		
		this->buttons.push_back(this->btnNaturalPlus);
		this->buttons.push_back(this->btnNatural);
		this->buttons.push_back(this->btnMedium);
		this->buttons.push_back(this->btnSurreal);
		this->buttons.push_back(this->btnSurrealPlus);
		
		
		this->sldStrength->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnSliderValueChanging), this->sldStrength));
		this->sldSaturation->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnSliderValueChanging), this->sldSaturation));
		this->sldLuminosity->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnSliderValueChanging), this->sldLuminosity));
		this->sldDetailContrast->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnSliderValueChanging), this->sldDetailContrast));
		this->sldLightingAdjustments->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnSliderValueChanging), this->sldLightingAdjustments));
		this->sldSmoothHighlights->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnSliderValueChanging), this->sldSmoothHighlights));
		this->sldGamma->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnSliderValueChanging), this->sldGamma));
		this->sldTemperature->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnSliderValueChanging), this->sldTemperature));
		this->sldMicroSmoothing->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnSliderValueChanging), this->sldMicroSmoothing));
		this->sldSaturationHighlights->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnSliderValueChanging), this->sldSaturationHighlights));
		this->sldSaturationShadows->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnSliderValueChanging), this->sldSaturationShadows));
		this->sldShadowsSmoothness->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnSliderValueChanging), this->sldShadowsSmoothness));
		this->sldShadowsClipping->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnSliderValueChanging), this->sldShadowsClipping));
		this->sldWhitePoint->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnSliderValueChanging), this->sldWhitePoint));
		this->sldBlackPoint->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &DetailsEnhancerSettings::OnSliderValueChanging), this->sldBlackPoint));
		
		// Update all sliders' values
		this->OnSliderValueChanging(this->sldStrength);
		this->OnSliderValueChanging(this->sldSaturation);
		this->OnSliderValueChanging(this->sldLuminosity);
		this->OnSliderValueChanging(this->sldDetailContrast);
		this->OnSliderValueChanging(this->sldLightingAdjustments);
		this->OnSliderValueChanging(this->sldSmoothHighlights);
		this->OnSliderValueChanging(this->sldGamma);
		this->OnSliderValueChanging(this->sldTemperature);
		this->OnSliderValueChanging(this->sldMicroSmoothing);
		this->OnSliderValueChanging(this->sldSaturationHighlights);
		this->OnSliderValueChanging(this->sldSaturationShadows);
		this->OnSliderValueChanging(this->sldShadowsSmoothness);
		this->OnSliderValueChanging(this->sldShadowsClipping);
		this->OnSliderValueChanging(this->sldWhitePoint);
		this->OnSliderValueChanging(this->sldBlackPoint);
	}

	DetailsEnhancerSettings::~DetailsEnhancerSettings()
	{
	}
	
	void DetailsEnhancerSettings::OnButtonClicked(Gtk::ToggleButton* btn)
	{
		// Prevent calling this handler for any state change
		if(!btn->get_active())
			return;
		
		for(int i = 0; i < this->buttons.size(); ++i)
		{
			if(this->buttons[i] != btn) this->buttons[i]->set_active(false);
		}
		
		ProcessingSettingsControl::OnSliderValueChanged();
	}
	
	void DetailsEnhancerSettings::OnLightingEffectsModeToggled()
	{
		this->lightingAdjustmentsSignal.block();
		
		this->chkLightingEffects->set_active(this->chkLightingEffectsMode->get_active());
		
		this->boxSlider->set_visible(!this->chkLightingEffectsMode->get_active());
		this->boxButtons->set_visible(this->chkLightingEffectsMode->get_active());
		
		this->lightingAdjustmentsSignal.unblock();
		
		this->OnSliderValueChanged();
	}
	
	void DetailsEnhancerSettings::OnLightingEffectsToggled()
	{
		this->effectsModeSignal.block();
		
		this->chkLightingEffectsMode->set_active(this->chkLightingEffects->get_active());
		
		this->boxSlider->set_visible(!this->chkLightingEffectsMode->get_active());
		this->boxButtons->set_visible(this->chkLightingEffectsMode->get_active());
		
		this->effectsModeSignal.unblock();
		
		this->OnSliderValueChanged();
	}
	
	PhotomatixImaging::XmpSettings DetailsEnhancerSettings::GetSettings()
	{
		XmpSettings settings;
		settings.parameters[0] = this->sldLuminosity->get_value();
		settings.parameters[1] = this->sldStrength->get_value();
		settings.parameters[2] = this->sldSaturation->get_value();
		settings.parameters[3] = this->sldWhitePoint->get_value();
		settings.parameters[4] = this->sldBlackPoint->get_value();
		if(this->btnNaturalPlus->get_active()) settings.parameters[5] = 0.0f;
		if(this->btnNatural->get_active()) settings.parameters[5] = 1.0f;
		if(this->btnMedium->get_active()) settings.parameters[5] = 2.0f;
		if(this->btnSurreal->get_active()) settings.parameters[5] = 3.0f;
		if(this->btnSurrealPlus->get_active()) settings.parameters[5] = 4.0f;
		
		settings.parameters[6] = this->sldDetailContrast->get_value();
		settings.parameters[7] = this->sldMicroSmoothing->get_value();
		settings.parameters[8] = this->sldGamma->get_value();
		settings.parameters[9] = this->sldSmoothHighlights->get_value();
		settings.parameters[10]= this->sldShadowsSmoothness->get_value();
		settings.parameters[11]= this->sldShadowsClipping->get_value();
		settings.parameters[12]= this->sldTemperature->get_value();
		settings.parameters[13]= this->sldSaturationHighlights->get_value();
		settings.parameters[14]= this->sldSaturationShadows->get_value();
		settings.parameters[15]= this->chk360image->get_active() ? 1 : 0;
		settings.parameters[16]= this->chkLightingEffects->get_active() ? 0 : 1;
		settings.parameters[17]= this->sldLightingAdjustments->get_value();
		
		settings.method = 1;
		
		return settings;
	}
	
	void DetailsEnhancerSettings::SetSettings(PhotomatixImaging::XmpSettings* settings)
	{
		this->sldLuminosity->set_value(settings->parameters[0]);
		this->sldStrength->set_value(settings->parameters[1]);
		this->sldSaturation->set_value(settings->parameters[2]);
		this->sldWhitePoint->set_value(settings->parameters[3]);
		this->sldBlackPoint->set_value(settings->parameters[4]);
		if(settings->parameters[5] == 0.0f) this->btnNaturalPlus->set_active(true);
		if(settings->parameters[5] == 1.0f) this->btnNatural->set_active(true);
		if(settings->parameters[5] == 2.0f) this->btnMedium->set_active(true);
		if(settings->parameters[5] == 3.0f) this->btnSurreal->set_active(true);
		if(settings->parameters[5] == 4.0f) this->btnSurrealPlus->set_active(true);
		
		this->sldDetailContrast->set_value(settings->parameters[6]);
		this->sldMicroSmoothing->set_value(settings->parameters[7]);
		this->sldGamma->set_value(settings->parameters[8]);
		this->sldSmoothHighlights->set_value(settings->parameters[9]);
		this->sldShadowsSmoothness->set_value(settings->parameters[10]);
		this->sldShadowsClipping->set_value(settings->parameters[11]);
		this->sldTemperature->set_value(settings->parameters[12]);
		this->sldSaturationHighlights->set_value(settings->parameters[13]);
		this->sldSaturationShadows->set_value(settings->parameters[14]);
		this->chk360image->set_active(settings->parameters[15]);
		

		this->lightingAdjustmentsSignal.block();
		this->effectsModeSignal.block();
		
			this->chkLightingEffectsMode->set_active(!settings->parameters[16]);
			this->chkLightingEffects->set_active(this->chkLightingEffectsMode->get_active());
		
			this->boxSlider->set_visible(!this->chkLightingEffectsMode->get_active());
			this->boxButtons->set_visible(this->chkLightingEffectsMode->get_active());
		
		this->lightingAdjustmentsSignal.unblock();
		this->effectsModeSignal.unblock();
		
		this->sldLightingAdjustments->set_value(settings->parameters[17]);
	}
	
	void DetailsEnhancerSettings::OnSliderValueChanging(Gtk::Scale* sender)
	{
		std::ostringstream str;
		str << std::fixed << std::setprecision(sender->get_digits()) << sender->get_value();
		
		if(sender == this->sldStrength) this->lblStrength->set_text(str.str());
		else if(sender == this->sldLuminosity) this->lblLuminosity->set_text(str.str());
		else if(sender == this->sldLightingAdjustments) this->lblLightingAdjustments->set_text(str.str());
		else if(sender == this->sldSaturation) this->lblSaturation->set_text(str.str());
		else if(sender == this->sldWhitePoint) this->lblWhitePoint->set_text(str.str());
		else if(sender == this->sldBlackPoint) this->lblBlackPoint->set_text(str.str());
		else if(sender == this->sldDetailContrast) this->lblDetailContrast->set_text(str.str());
		else if(sender == this->sldMicroSmoothing) this->lblMicroSmoothing->set_text(str.str());
		else if(sender == this->sldGamma) this->lblGamma->set_text(str.str());
		else if(sender == this->sldSmoothHighlights) this->lblSmoothHighlights->set_text(str.str());
		else if(sender == this->sldShadowsSmoothness) this->lblShadowsSmoothness->set_text(str.str());
		else if(sender == this->sldShadowsClipping) this->lblShadowsClipping->set_text(str.str());
		else if(sender == this->sldTemperature) this->lblTemperature->set_text(str.str());
		else if(sender == this->sldSaturationHighlights) this->lblSaturationHighlights->set_text(str.str());
		else if(sender == this->sldSaturationShadows) this->lblSaturationShadows->set_text(str.str());
	}

}