#include "../Photomatix32/Photomatix32.h"
#include "../Photomatix32/Photomatix40Lib1.h"

#include <iostream>
#include <algorithm>

#include "displayableimagecontrol.h"
#include "../PhotomatixImaging/photoimage.h"

#include "cairomm/context.h"
#include "gdkmm/general.h"
#include <gtkmm/scrollbar.h>
#include <gtkmm/adjustment.h>


using namespace std;
using namespace PhotomatixImaging;


namespace PhotomatixControls
{
	DisplayableImageControl::DisplayableImageControl():
		DisplayableImage()
	{
		this->set_has_window(false);
		this->set_redraw_on_allocate(false);
		this->set_border_width(0);
	}

	DisplayableImageControl::DisplayableImageControl(const string& fileName):
		DisplayableImage(fileName)
	{
		this->set_has_window(false);
		this->set_redraw_on_allocate(false);
		this->set_border_width(0);
	}
	
	DisplayableImageControl::~DisplayableImageControl()
	{
	}
	
	void DisplayableImageControl::Open(const string& fileName, bool openRAWs, int colorTemperature, int colorSpace)
	{
		DisplayableImage::Open(fileName, openRAWs, colorTemperature, colorSpace);
		this->add(*this->image);
	}

	void DisplayableImageControl::SetPhoto(shared_ptr< PhotoImage > photo)
	{
		DisplayableImage::SetPhoto(photo);
		this->add(*this->image);
	}
	
	void DisplayableImageControl::SetZoomFactor(float zoomFactor, int maxWidth, int maxHeight)
	{
		float zoom = zoomFactor >= 1.0 ? zoomFactor : 1.0f;
		
		int width = maxWidth ? maxWidth : this->photo->GetWidth() * zoom;
		int height = maxHeight ? maxHeight : this->photo->GetHeight() * zoom;
		
		width = this->photo->GetWidth() * zoomFactor;
		height = this->photo->GetHeight() * zoomFactor;
		
		// Image is larger than control size
		if(0 && this->setUp && (width > this->get_width() && height > this->get_height()))
		{
			int width = maxWidth ? maxWidth / zoomFactor : this->photo->GetWidth() / zoomFactor;
			int height = maxHeight ? maxHeight / zoomFactor : this->photo->GetHeight() / zoomFactor;
			
			Glib::RefPtr <Gdk::Pixbuf> pixbuf = Gdk::Pixbuf::create_subpixbuf(this->pixBuf, 0, 0, width, height);
			
			this->image->SetScale(zoomFactor);
			
			this->image->SetPixBuf(pixbuf);
		}
		// Image is smaller than control size
		else
		{
			this->image->SetScale(zoomFactor);
			
			Glib::RefPtr <Gdk::Pixbuf> pixbuf = this->pixBuf;
			image->SetPixBuf(pixbuf);
		}
		
		//this->set_size_request(maxWidth ? maxWidth : this->photo->GetWidth() * zoomFactor, maxHeight ? maxHeight : this->photo->GetHeight() * zoomFactor);

		this->image->set_size_request(this->photo->GetWidth() * zoomFactor, this->photo->GetHeight() * zoomFactor);
																																																												
		this->image->queue_draw();
		
		this->previousHeight = maxHeight;
		this->previousWidth = maxWidth;
		this->previousZoom = zoomFactor;
		
		this->setUp = true;
	}
}