#include "exposurefusionsettings.h"
#include "../PhotomatixExternalLibs/stlsoft/include/stlsoft/string/simple_string.hpp"
#include <iostream>
#include <iomanip>

using namespace Photomatix;
using namespace PhotomatixImaging;


namespace PhotomatixControls
{
	ExposureFusionSettings::ExposureFusionSettings(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder):
		ProcessingSettingsControl(cobject, builder)
	{
		// Set up sliders
		builder->get_widget("sldStrength", this->sldStrength);
		builder->get_widget("sldBlendingPoint", this->sldBlendingPoint);
		builder->get_widget("sldShadows", this->sldShadows);
		builder->get_widget("sldLocalContrast", this->sldLocalContrast);
		builder->get_widget("sldColorSaturation", this->sldColorSaturation);
		builder->get_widget("sldWhiteClip", this->sldWhiteClip);
		builder->get_widget("sldBlackClip", this->sldBlackClip);
		builder->get_widget("sldMidtone", this->sldMidtone);
		builder->get_widget("sldTemperature", this->sldTemperature);
		
		builder->get_widget("lblStrength", this->lblStrength);		
		builder->get_widget("lblBlendingPoint", this->lblBlendingPoint);
		builder->get_widget("lblShadows", this->lblShadows);
		builder->get_widget("lblLocalContrast", this->lblLocalContrast);
		builder->get_widget("lblColorSaturation", this->lblColorSaturation);
		builder->get_widget("lblWhiteClip", this->lblWhiteClip);
		builder->get_widget("lblBlackClip", this->lblBlackClip);
		builder->get_widget("lblMidtone", this->lblMidtone);
		builder->get_widget("lblTemperature", this->lblTemperature);
		
		
		this->sldStrength->set_range(-10.0, 10.0);
		this->sldBlendingPoint->set_range(-10.0, 10.0);
		this->sldShadows->set_range(0.0, 10.0);
		this->sldLocalContrast->set_range(0.0, 10.0);
		this->sldColorSaturation->set_range(-10.0, 10.0);
		this->sldTemperature->set_range(-10.0, 10.0);
		this->sldWhiteClip->set_range(0.0, 10.0);
		this->sldBlackClip->set_range(0.0, 10.0);
		this->sldMidtone->set_range(-10.0, 10.0);
		
		this->sldLocalContrast->set_value(2.0f);
		
		this->sldStrength->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldBlendingPoint->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldShadows->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldLocalContrast->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldColorSaturation->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldWhiteClip->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldBlackClip->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldMidtone->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldTemperature->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		
		this->sldStrength->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ExposureFusionSettings::OnSliderValueChanging), this->sldStrength));
		this->sldBlendingPoint->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ExposureFusionSettings::OnSliderValueChanging), this->sldBlendingPoint));
		this->sldShadows->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ExposureFusionSettings::OnSliderValueChanging), this->sldShadows));
		this->sldLocalContrast->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ExposureFusionSettings::OnSliderValueChanging), this->sldLocalContrast));
		this->sldColorSaturation->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ExposureFusionSettings::OnSliderValueChanging), this->sldColorSaturation));
		this->sldWhiteClip->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ExposureFusionSettings::OnSliderValueChanging), this->sldWhiteClip));
		this->sldBlackClip->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ExposureFusionSettings::OnSliderValueChanging), this->sldBlackClip));
		this->sldMidtone->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ExposureFusionSettings::OnSliderValueChanging), this->sldMidtone));
		this->sldTemperature->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ExposureFusionSettings::OnSliderValueChanging), this->sldTemperature));
		
		// Update all sliders' values
		this->OnSliderValueChanging(this->sldStrength);
		this->OnSliderValueChanging(this->sldBlendingPoint);
		this->OnSliderValueChanging(this->sldShadows);
		this->OnSliderValueChanging(this->sldLocalContrast);
		this->OnSliderValueChanging(this->sldColorSaturation);
		this->OnSliderValueChanging(this->sldWhiteClip);
		this->OnSliderValueChanging(this->sldBlackClip);
		this->OnSliderValueChanging(this->sldMidtone);
		this->OnSliderValueChanging(this->sldTemperature);
	}

	ExposureFusionSettings::~ExposureFusionSettings()
	{
	}
	
	XmpSettings ExposureFusionSettings::GetSettings()
	{
		XmpSettings settings;
		settings.parameters[0] = this->sldStrength->get_value();
		settings.parameters[1] = this->sldBlendingPoint->get_value();
		settings.parameters[2] = this->sldShadows->get_value();
		settings.parameters[3] = this->sldColorSaturation->get_value();
		settings.parameters[4] = this->sldWhiteClip->get_value();
		settings.parameters[5]= this->sldBlackClip->get_value();
		settings.parameters[6]= this->sldMidtone->get_value();
		settings.parameters[8]= this->sldLocalContrast->get_value();
		settings.parameters[12]= this->sldTemperature->get_value();
		
		settings.method = 10;
		
		return settings;
	}
	
	void ExposureFusionSettings::SetSettings(XmpSettings* settings)
	{
		this->sldStrength->set_value(settings->parameters[0]);
		this->sldBlendingPoint->set_value(settings->parameters[1]);
		this->sldShadows->set_value(settings->parameters[2]);
		this->sldColorSaturation->set_value(settings->parameters[3]);
		this->sldWhiteClip->set_value(settings->parameters[4]);
		this->sldBlackClip->set_value(settings->parameters[5]);
		this->sldMidtone->set_value(settings->parameters[6]);
		this->sldLocalContrast->set_value(settings->parameters[8]);
		this->sldTemperature->set_value(settings->parameters[12]);
	}
	
	void ExposureFusionSettings::OnSliderValueChanging(Gtk::Scale* sender)
	{
		std::ostringstream str;
		str << std::fixed << std::setprecision(sender->get_digits()) << sender->get_value();
		
		if(sender == this->sldStrength) this->lblStrength->set_text(str.str());
		else if(sender == this->sldBlendingPoint) this->lblBlendingPoint->set_text(str.str());
		else if(sender == this->sldShadows) this->lblShadows->set_text(str.str());
		else if(sender == this->sldLocalContrast) this->lblLocalContrast->set_text(str.str());
		else if(sender == this->sldColorSaturation) this->lblColorSaturation->set_text(str.str());
		else if(sender == this->sldWhiteClip) this->lblWhiteClip->set_text(str.str());
		else if(sender == this->sldBlackClip) this->lblBlackClip->set_text(str.str());
		else if(sender == this->sldMidtone) this->lblMidtone->set_text(str.str());
		else if(sender == this->sldTemperature) this->lblTemperature->set_text(str.str());
	}

}