#include "../Photomatix32/Photomatix32.h"
#include "../Photomatix32/Photomatix40Lib1.h"

#include <iostream>
#include <algorithm>

#include "scrollabledisplayableimagecontrol.h"
#include "../PhotomatixImaging/photoimage.h"

#include "cairomm/context.h"
#include "gdkmm/general.h"
#include <gtkmm/scrollbar.h>
#include <gtkmm/adjustment.h>


using namespace std;
using namespace PhotomatixImaging;


namespace PhotomatixControls
{
	void ScrollableDisplayableImageControl::ShowScrollbars(bool show)
	{
		if(show)
			this->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
		else
			this->set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_NEVER);
	}
	
	ScrollableDisplayableImageControl::ScrollableDisplayableImageControl():
		DisplayableImage()
	{
		//this->set_has_window(false);
		this->set_redraw_on_allocate(false);
		this->set_border_width(0);
		this->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
		this->get_vscrollbar()->signal_value_changed().connect(sigc::mem_fun(*this, &ScrollableDisplayableImageControl::OnScrollbarValueChanged));
		this->get_hscrollbar()->signal_value_changed().connect(sigc::mem_fun(*this, &ScrollableDisplayableImageControl::OnScrollbarValueChanged));
	}

	ScrollableDisplayableImageControl::ScrollableDisplayableImageControl(const string& fileName):
		DisplayableImage(fileName)
	{
		//this->set_has_window(false);
		this->set_redraw_on_allocate(false);
		this->set_border_width(0);
		this->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
		this->get_vscrollbar()->signal_value_changed().connect(sigc::mem_fun(*this, &ScrollableDisplayableImageControl::OnScrollbarValueChanged));
		this->get_hscrollbar()->signal_value_changed().connect(sigc::mem_fun(*this, &ScrollableDisplayableImageControl::OnScrollbarValueChanged));
	}
	
	ScrollableDisplayableImageControl::~ScrollableDisplayableImageControl()
	{
	}

	void ScrollableDisplayableImageControl::OnScrollbarValueChanged()
	{
		this->SetZoomFactor(this->previousZoom, this->previousWidth, this->previousHeight);
	}
	
	void ScrollableDisplayableImageControl::Open(const string& fileName, bool openRAWs, int colorTemperature, int colorSpace)
	{
		DisplayableImage::Open(fileName, openRAWs, colorTemperature, colorSpace);
		this->add(*this->image);
		//this->add(aa);
		//this->show_all_children();
	}

	void ScrollableDisplayableImageControl::SetPhoto(shared_ptr< PhotoImage > photo)
	{
		DisplayableImage::SetPhoto(photo);
		this->add(*this->image);
		//this->add(aa);
		//this->show_all_children();
	}
	
	void ScrollableDisplayableImageControl::RemoveChildControl()
	{
		this->remove();
	}
	
	void ScrollableDisplayableImageControl::SetZoomFactor(float zoomFactor, int maxWidth, int maxHeight)
	{
		float zoom = zoomFactor >= 1.0 ? zoomFactor : 1.0f;
		
		int width = maxWidth ? maxWidth : this->photo->GetWidth() * zoom;
		int height = maxHeight ? maxHeight : this->photo->GetHeight() * zoom;
		
		width = this->photo->GetWidth() * zoomFactor;
		height = this->photo->GetHeight() * zoomFactor;
		
		// Image is larger than control size
		if(0 && this->setUp && (width > this->get_width() && height > this->get_height()))
		{
			int width = maxWidth ? maxWidth / zoomFactor : this->photo->GetWidth() / zoomFactor;
			int x = this->get_hscrollbar()->get_value() / zoomFactor;
			if(x + width > this->pixBuf->get_width())
				x = this->pixBuf->get_width() - width;
			int height = maxHeight ? maxHeight / zoomFactor : this->photo->GetHeight() / zoomFactor;
			int y = this->get_vscrollbar()->get_value() / zoomFactor;
			if(y + height > this->pixBuf->get_height())
				y = this->pixBuf->get_height() - height;
			Glib::RefPtr <Gdk::Pixbuf> pixbuf = Gdk::Pixbuf::create_subpixbuf(this->pixBuf, x, y, width, height);
			
			this->image->SetScale(zoomFactor);
			
			this->image->SetPixBuf(pixbuf);
			this->image->SetOffset(this->get_hscrollbar()->get_value(), this->get_vscrollbar()->get_value());
		}
		// Image is smaller than control size
		else
		{
			this->image->SetScale(zoomFactor);
			
			Glib::RefPtr <Gdk::Pixbuf> pixbuf = this->pixBuf;
			//Glib::RefPtr <Gdk::Pixbuf> pixbuf = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, false, 8, width, height);
			//this->pixBuf->scale(pixbuf, 0, 0, width, height, 0, 0, zoomFactor, zoomFactor, Gdk::INTERP_BILINEAR);
			image->SetPixBuf(pixbuf);
			
			// We draw image in the middle of the control in this case
			if(this->setUp)
				this->image->SetOffset(-this->get_hscrollbar()->get_value(), -this->get_vscrollbar()->get_value());// (this->get_height() - height) / 2);
		}
		
		//this->set_size_request(maxWidth ? maxWidth : this->photo->GetWidth() * zoomFactor, maxHeight ? maxHeight : this->photo->GetHeight() * zoomFactor);

		this->image->set_size_request(this->photo->GetWidth() * zoomFactor, this->photo->GetHeight() * zoomFactor);
																																																												
		this->image->queue_draw();
		
		this->previousHeight = maxHeight;
		this->previousWidth = maxWidth;
		this->previousZoom = zoomFactor;
		
		this->setUp = true;
	}

}