#ifndef SCROLLABLEDISPLAYABLEIMAGECONTROL_H
#define SCROLLABLEDISPLAYABLEIMAGECONTROL_H

#include <gtkmm/enums.h>
#include <gtkmm/container.h>
#include <gtkmm/image.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/box.h>
#include <gtkmm/drawingarea.h>
#include <gtkmm/label.h>
#include <memory>
#include "displayableimage.h"


namespace PhotomatixImaging
{
	class PhotoImage;
}

namespace PhotomatixControls
{
	/** Subclass of DisplayableImageControl allowing to display an image inside scrollable window.
	 */
	class ScrollableDisplayableImageControl : public Gtk::ScrolledWindow, public DisplayableImage
	{
	public:
		ScrollableDisplayableImageControl();
		ScrollableDisplayableImageControl(const std::string& fileName);
		virtual ~ScrollableDisplayableImageControl();
		
		/** Specifies if the scrollbars should be shown.
		 * @param show if false then scrollbars will never be shown. If it is true scrollbars
		 * will be shown only when they are needed (i.e. when contents of this controls is larger
		 * than its size).
		 */
		void ShowScrollbars(bool show);
		
		/** Creates new PhotoImage object by loading it from file. Then this image is displayed
		 * by the control.
		 */
		void Open(const std::string& fileName, bool openRAWs=false, int colorTemperature=0, int colorSpace=0);
		
		/// Method that allows showing existing PhotoImage object.
		void SetPhoto(std::shared_ptr <PhotomatixImaging::PhotoImage> photo);
		
		void SetZoomFactor(float zoomFactor, int maxWidth=0, int maxHeight=0);
		
		void RemoveChildControl();
		
	protected:
		void OnScrollbarValueChanged();
	};
}

#endif //GTKMM_CUSTOM_CONTAINER_MYCONTAINER_H
