#include "tonebalancersettings.h"
#include <gtkmm-3.0/gtkmm/label.h>
#include <iostream>
#include <iomanip>

using namespace Photomatix;
using namespace PhotomatixImaging;


namespace PhotomatixControls
{
	ToneBalancerSettings::ToneBalancerSettings(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder):
		ProcessingSettingsControl(cobject, builder),
		scalingFloor(0.0f),
		scalingCeil(0.0f)
	{
		this->ClipPowerOf2 = (float)(log(0.025) / log(0.3));
		
		
		// Set up sliders
		builder->get_widget("sldStrength", this->sldStrength);
		builder->get_widget("sldLighting", this->sldLighting);
		builder->get_widget("sldBrightnessBoost", this->sldBrightnessBoost);
		builder->get_widget("sldMicrocontrast", this->sldMicrocontrast);
		builder->get_widget("sldWhitePoint", this->sldWhitePoint);
		builder->get_widget("sldBlackPoint", this->sldBlackPoint);
		builder->get_widget("sldTemperature", this->sldTemperature);
		builder->get_widget("sldSaturation", this->sldSaturation);
		builder->get_widget("sldBrightness", this->sldBrightness);
		
		builder->get_widget("lblStrength", this->lblStrength);
		builder->get_widget("lblLighting", this->lblLighting);
		builder->get_widget("lblBrightnessBoost", this->lblBrightnessBoost);
		builder->get_widget("lblMicrocontrast", this->lblMicrocontrast);
		builder->get_widget("lblWhitePoint", this->lblWhitePoint);
		builder->get_widget("lblBlackPoint", this->lblBlackPoint);
		builder->get_widget("lblTemperature", this->lblTemperature);
		builder->get_widget("lblSaturation", this->lblSaturation);
		builder->get_widget("lblBrightness", this->lblBrightness);
		
		
		this->sldStrength->set_range(0, 100);
		this->sldStrength->set_digits(0);
		this->sldLighting->set_range(-10.0, 10.0);
		this->sldBrightnessBoost->set_range(0, 100);
		this->sldBrightnessBoost->set_digits(0);
		this->sldMicrocontrast->set_range(-10.0, 10.0);
		this->sldWhitePoint->set_range(0.0, 10.0);
		this->sldBlackPoint->set_range(0.0, 10.0);
		this->sldTemperature->set_range(-10.0, 10.0);
		this->sldSaturation->set_range(-10.0, 10.0);
		this->sldBrightness->set_range(-10.0, 10.0);
		
		this->sldStrength->set_value(50);
		this->sldWhitePoint->set_value(4.0);
		
		this->sldStrength->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldLighting->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldBrightnessBoost->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldMicrocontrast->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldWhitePoint->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldBlackPoint->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldTemperature->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldSaturation->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		this->sldBrightness->signal_button_release_event().connect(sigc::mem_fun(*this, &ProcessingSettingsControl::OnMouseUp), false);
		
		this->sldStrength->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ToneBalancerSettings::OnSliderValueChanging), this->sldStrength));
		this->sldLighting->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ToneBalancerSettings::OnSliderValueChanging), this->sldLighting));
		this->sldBrightnessBoost->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ToneBalancerSettings::OnSliderValueChanging), this->sldBrightnessBoost));
		this->sldMicrocontrast->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ToneBalancerSettings::OnSliderValueChanging), this->sldMicrocontrast));
		this->sldWhitePoint->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ToneBalancerSettings::OnSliderValueChanging), this->sldWhitePoint));
		this->sldBlackPoint->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ToneBalancerSettings::OnSliderValueChanging), this->sldBlackPoint));
		this->sldTemperature->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ToneBalancerSettings::OnSliderValueChanging), this->sldTemperature));
		this->sldSaturation->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ToneBalancerSettings::OnSliderValueChanging), this->sldSaturation));
		this->sldBrightness->signal_value_changed().connect(sigc::bind<Gtk::Scale*>(sigc::mem_fun(*this, &ToneBalancerSettings::OnSliderValueChanging), this->sldBrightness));
		
		// Update all sliders' values
		this->OnSliderValueChanging(this->sldStrength);
		this->OnSliderValueChanging(this->sldLighting);
		this->OnSliderValueChanging(this->sldBrightnessBoost);
		this->OnSliderValueChanging(this->sldMicrocontrast);
		this->OnSliderValueChanging(this->sldWhitePoint);
		this->OnSliderValueChanging(this->sldBlackPoint);
		this->OnSliderValueChanging(this->sldTemperature);
		this->OnSliderValueChanging(this->sldSaturation);
		this->OnSliderValueChanging(this->sldBrightness);
	}

	ToneBalancerSettings::~ToneBalancerSettings()
	{
	}
	
	XmpSettings ToneBalancerSettings::GetSettings()
	{
		XmpSettings settings;
		settings.parameters[0] = this->sldStrength->get_value();
		settings.parameters[1] = this->sldLighting->get_value();
		settings.parameters[2] = this->sldBrightnessBoost->get_value();
		settings.parameters[5] = this->sldMicrocontrast->get_value();
		settings.parameters[8] = this->sldBrightness->get_value();
		settings.parameters[12] = this->sldTemperature->get_value();
		settings.parameters[13] = this->sldSaturation->get_value();
		settings.parameters[17] = this->scalingFloor;
		settings.parameters[18] = this->scalingCeil;
		
		float valPower = this->ClipPowerOf2;
		settings.parameters[3] = 10.0f * powf(this->sldWhitePoint->get_value() / 10.0, valPower);
		settings.parameters[4] = 10.0f * powf(this->sldBlackPoint->get_value() / 10.0, valPower);
		
		settings.method = 4;
		
		return settings;
	}
	
	void ToneBalancerSettings::SetSettings(XmpSettings* settings)
	{
		this->sldStrength->set_value(settings->parameters[0]);
		this->sldLighting->set_value(settings->parameters[1]);
		this->sldBrightnessBoost->set_value(settings->parameters[2]);
		this->sldMicrocontrast->set_value(settings->parameters[5]);
		this->sldBrightness->set_value(settings->parameters[8]);
		this->sldTemperature->set_value(settings->parameters[12]);
		this->sldSaturation->set_value(settings->parameters[13]);
		//settings.parameters[17] = this->scalingFloor;
		//settings.parameters[18] = this->scalingCeil;
		
		float invPower = 1.0f / this->ClipPowerOf2;
		this->sldWhitePoint->set_value(10.0f * powf(settings->parameters[3] / 10.0f, invPower));
		this->sldBlackPoint->set_value(10.0f * powf(settings->parameters[4] / 10.0f, invPower));
		
	}
	
	void ToneBalancerSettings::OnSliderValueChanging(Gtk::Scale* sender)
	{
		std::ostringstream str;
		str << std::fixed << std::setprecision(sender->get_digits()) << sender->get_value();
		
		if(sender == this->sldStrength) this->lblStrength->set_text(str.str());
		else if(sender == this->sldLighting) this->lblLighting->set_text(str.str());
		else if(sender == this->sldBrightnessBoost) this->lblBrightnessBoost->set_text(str.str());
		else if(sender == this->sldMicrocontrast) this->lblMicrocontrast->set_text(str.str());
		else if(sender == this->sldWhitePoint) this->lblWhitePoint->set_text(str.str());
		else if(sender == this->sldBlackPoint) this->lblBlackPoint->set_text(str.str());
		else if(sender == this->sldTemperature) this->lblTemperature->set_text(str.str());
		else if(sender == this->sldSaturation) this->lblSaturation->set_text(str.str());
		else if(sender == this->sldBrightness) this->lblBrightness->set_text(str.str());
	}

}