#ifndef EXPOSUREFUSIONPROCESSOR_H
#define EXPOSUREFUSIONPROCESSOR_H


#include "photomerger.h"
#include "photomatixconstants.h"


namespace PhotomatixImaging
{
	/// Type of the exposure fusion
	enum EFusionMethod
	{
		FusionAdjust=10,
		FusionIntensive=11,
		FusionRealistic=12,
		FusionTwoImages=13,
		FusionAverage=14,
		FusionAuto=15
	};

	class ExposureFusionSettings
	{
	protected:
		float fusionParams[PhotomatixMisc::XmpParamsLength];

	public:
		ExposureFusionSettings() {memset(fusionParams, 0, sizeof(float) * PhotomatixMisc::XmpParamsLength);};
		
		float* GetSettings() {return fusionParams;}

		bool IsAdjust360() const {return (fusionParams[7] > 0);}

		float GetRealisticColorSaturation() const {return fusionParams[3];}

		float GetIntensiveStrength() const {return fusionParams[0];}
		float GetIntensiveColorSaturation() const {return fusionParams[1];}
		float GetIntensiveRadius() const {return fusionParams[2];}
		void SetIntensiveRadius(float radius) {fusionParams[2] = radius;}

		int GetTwoImagesImage1() {return (int)fusionParams[0];}
		void SetTwoImagesImage1(int index) {fusionParams[0] = index;}
		int GetTwoImagesImage2() {return (int)fusionParams[1];}
		void SetTwoImagesImage2(int index) {fusionParams[1] = index;}
	};

	/// Processor for fusing images.
	class ExposureFusionProcessor: public PhotoMerger
	{
	protected:
		float tmParams[PhotomatixMisc::XmpParamsLength];
		
		EFusionMethod fusionMethod;
		ExposureFusionSettings currentSettings;

		void ProcessPhoto();

	public:
		ExposureFusionProcessor();
		~ExposureFusionProcessor();

		void SetExposureFusionMethod(EFusionMethod type) {this->fusionMethod = type;}
		void SetCurrentSettings(ExposureFusionSettings& settings) {this->currentSettings = settings;}
		
		/// @TODO: For now we just array but may introduce XmpSettings class instead...
		void SetProcessingSettings(float* params);
	};
}


#endif // BLENDER_H
