#ifndef PHOTOMERGER_H
#define PHOTOMERGER_H


#include "photoprocessor.h"
#include <vector>


namespace PhotomatixImaging
{
	/// Type of the exposure data
	enum EExposureInfo
	{
		Standard,													///< Exposure data is correct
		Same,														///< Photos of a bracketed sequence have the same exposure
		None														///< Exposure data is missing
	};


	/** PhotoMerger class is a base class for Blender and HdrGenerator
	*/
	class PhotoMerger: public PhotoProcessor
	{
	protected:
		float ghostingDetection;

		bool reduceChromaticAberrations;							///< Is chromatic aberrations reduction enabled?
		bool unghostImages;											///< Is ghost removal enabled?


		std::shared_ptr<std::vector <unsigned char> > ghostMask;	///< Ghost mask
		std::vector <int> indexFixedExpo;

		
		/// Processes photo.
		virtual void ProcessPhoto()=0;

		bool IsExifFound();
		
		/// Detects ghosting in the photo.
		void DetectGhosting(std::vector <float>& delta);

		/** Checks if the exposure data of input photos is valid.
		 * @returns EExposureInfo - if exposure data is correct EExposureInfo::Standard is returned.
		 * If it is the same for two or more photos - EExposureInfo::Same and if it is missing - EExposureInfo::None.
		 */ 
		EExposureInfo CheckExposureData();

		float GetEVSpacing(std::vector <float>& delta);

	public:
		PhotoMerger();
		virtual ~PhotoMerger();

		std::shared_ptr<std::vector <unsigned char> > GetGhostMask() const {return this->ghostMask;}
		void SetGhostMask(std::shared_ptr<std::vector <unsigned char> > ghostMask) {this->ghostMask = ghostMask;}

		std::vector <int> GetIndexFixedExpo() const {return this->indexFixedExpo;}
		void SetIndexFixedExpo(std::vector <int> indexFixedExpo) {this->indexFixedExpo = indexFixedExpo;}

		void SetGhostRemovalEnabled(bool enabled) {this->unghostImages = enabled;}
		bool IsGhostRemovalEnabled() const {return this->unghostImages;}
		void SetGhostRemovalDetectionStrength(float detection) {this->ghostingDetection = detection;}
		float GetGhostRemovalDetectionStrength() const {return this->ghostingDetection;}

		void SetReduceChromaticAberrationsEnabled(bool reduceCA) {this->reduceChromaticAberrations = reduceCA;}
		bool IsReduceChromaticAberrationEnabled() const {return this->reduceChromaticAberrations;}
	};
}


#endif // PHOTOMERGER_H
