#ifndef PRESET_H
#define PRESET_H


#include <cstring>
#include <string>

namespace PhotomatixImaging
{
  struct XmpSettings
  {
	  float parameters[60];                   /// TODO: use constant defined in PhotomatixConstants.cpp instead of 20
	  int method;                                             /// TODO: introduce enum for that
	  
	  XmpSettings() {
		  memset(parameters, 0, sizeof(float) * 60);
		  method = 0;
	  }
  };
  
  /** XmpSettings for Presets.
    */
  class Preset
  {
  protected:
	  std::string path;
	  std::string displayName;

	  XmpSettings settings;
	  
  public:
	  Preset() {}
	  Preset(const std::string& displayName, const std::string& path, bool isBuiltIn=true, int method=-1);
	  virtual ~Preset() {}
	  
	  static void GetMethodDefault(int method, int docType, float* tmParams);
	  
	  void LoadPresetFromFile(const std::string& fileName);
	  void SetDefault(int method);
	  
	  /// TODO: we might also overload == operator instead
	  bool ComparePresets(Preset& preset) {return false;}
	  
	  /// Returns processing method.
	  int GetMethod() const {return this->settings.method;}
	  
	  /// Returns settings
	  XmpSettings* GetSettings() {return &this->settings;}
	  
	  
	  /// Returns display name for this preset.
	  std::string& GetDisplayName() {return this->displayName;}
  };
  
}


#endif