#ifndef TONEMAPPER_H
#define TONEMAPPER_H


#include "photoprocessor.h"
#include "../PhotomatixMisc/photomatixconstants.h"
#include <vector>
#include <memory>


namespace PhotomatixImaging
{
	enum EToneMappingMethod
	{
		TM_DETAILS_ENHANCER = 1,
		TM_TONE_COMPRESSOR  = 2,
		TM_CONTRAST_OPTIMIZER = 3,
		TM_TONE_BALANCER = 4
	};
	
	/** Tone-maps photo using either Details Enhancer or Tone Compressor algorithm.
	 */
	class ToneMapper: public PhotoProcessor
	{
	protected:
		PhotomatixImaging::EToneMappingMethod method;
		float tmParams[PhotomatixMisc::XmpParamsLength];
		
		std::shared_ptr <std::vector <char>> contrastOptimizerWorkBuffer;
		
		float contrastOptimizerDynamicRange;
		
		float toneBalancerScalingFloor;
		float toneBalancerScalingCeil;
		
		bool isContrastOptimizerAlreadyMapped;
		
		
		virtual void ProcessPhoto();
		
		std::shared_ptr <std::vector <unsigned char>> DetailsEnhancer();
		std::shared_ptr <std::vector <unsigned char>> ToneCompressor();
		std::shared_ptr <std::vector <unsigned char>> ToneBalancer();
		std::shared_ptr <std::vector <unsigned char>> ContrastOptimizer();
		
	public:
		ToneMapper();
		virtual ~ToneMapper();
		
		void SetToneMappingMethod(PhotomatixImaging::EToneMappingMethod meth) {this->method = meth;}
		PhotomatixImaging::EToneMappingMethod GetToneMappingMethod() const {return this->method;}
		/// @TODO: For now we just array but may introduce XmpSettings class instead...
		void SetProcessingSettings(float* params);
		
		void SetContrastOptimizerAlreadyMapped(bool value) {this->isContrastOptimizerAlreadyMapped = value;}
	};
}

	
#endif // TONEMAPPER_H
