#ifndef RESOURCEUTILS_H
#define RESOURCEUTILS_H

#ifndef WIN32
	#include <unistd.h>
	#include <libgen.h>
#endif
#include <climits>
#include "Exceptions.h"


namespace PhotomatixUtils
{
#ifndef WIN32
	///
	/// Gets the directory of the executable
	///
	inline std::string GetApplicationDir()
	{
		char str[1024];
		int len = readlink("/proc/self/exe", str, sizeof(str));
		if (len < -1) throw Photomatix::ResourceNotFoundException("Error reading /proc/self/exe");
		str[len] = 0;
		return dirname(str);
	}
	
	///
	/// Returns the full path name for the Glade resource given
	//
	inline std::string GetGladeResourcePath(const std::string &resName)
	{
#ifdef Release
		return "/usr/share/photomatix/" + resName + ".glade";
#else
		return GetApplicationDir() + "/" + resName + ".glade";
#endif
	}
#endif
	
	///
	/// Returns common part of two strings
	///
	inline std::string GetStringCommonPart(std::string text1, std::string text2, unsigned int minSeuilLen)
	{
		std::string commonPart = "";
		unsigned int i = 1, lenFicMin = text1.length(); 
		if(text2.length() < lenFicMin)
			lenFicMin=text2.length();
		
		while (i <= lenFicMin)
		{
			if (text1.substr(0,i) != text2.substr(0,i))
				break;
			++i;
		}
		if(i > minSeuilLen)
			commonPart = text1.substr(0,i-1);
		return commonPart;
	}
}
#endif
