#ifndef WORKERTHREAD_H
#define WORKERTHREAD_H

#ifdef WEBSERVER
#include <pthread.h>
#else
#include <gtkmm.h>
#endif

namespace PhotomatixMisc
{
	/// WorkerThread class wrapping Glib::Thread class.
	class WorkerThread
	{
	protected:
#ifndef WEBSERVER
		Glib::Thread* thread;
		Glib::Mutex mutex;
#endif
		
		bool stop;
		bool cancelled;
		
		
		/// This is where the real work happens.
		void Run();
		
		virtual void Process()=0;
		
	public:
		WorkerThread();
		~WorkerThread();
		
		/// Called to start the processing on the thread.
		virtual void Start();
		
		/// Stops thread execution.
		void Stop();

#ifndef WEBSERVER		
		/// Signal emitted when thread finishes processing.
		Glib::Dispatcher sig_done;
#endif
		
		bool Cancelled() const {return this->cancelled;}
		void Reset() {this->cancelled = this->stop = false;}
	};
}

#endif // WORKERTHREAD_H
